// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Jul 14 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ArtCompressibleBSplitFlux2D_HH
#define ArtCompressibleBSplitFlux2D_HH

#include <mouse_config.h>

class ArtCompressibleBSplitFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <IncompressibleOperator2D.hh>
#include <ArtCompressibleRoe2D.hh>

/**
*/
class ArtCompressibleBSplitFlux2D : public IncompressibleOperator2D
{
protected:

public:
  /** */ ArtCompressibleBSplitFlux2D (carg_t arg);
  /** */ virtual ~ArtCompressibleBSplitFlux2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void ArtCompressibleBSplitFlux2D::operate(size_t ie, size_t ip, 
						 size_t from, size_t to, 
						 real nx, real ny, 
						 bool mod_from, bool mod_to)
{
  real flux_p, flux_u, flux_v;
  if (mod_from) {
    real p_cell = p[from];
    real u_cell = u[from];
    real v_cell = v[from];
    //p_cell = p_oo;
    //u_cell = u_oo;
    //v_cell = v_oo;
    real p_av = .5*(p_cell + p_oo);
    real u_av = .5*(u_cell + u_oo);
    real v_av = .5*(v_cell + v_oo);
    ArtCompressibleRoe2D::compute(p_oo, u_oo, v_oo,
				  p_av, u_av, v_av,
				  p_cell, u_cell, v_cell,
				  nx, ny, beta_q,
				  flux_p, flux_u, flux_v);
    res_p[from] += flux_p;
    res_u[from] += flux_u;
    res_v[from] += flux_v;
  };
  if (mod_to) {
    real p_cell = p[to];
    real u_cell = u[to];
    real v_cell = v[to];
    //p_cell = p_oo;
    //u_cell = u_oo;
    //v_cell = v_oo;
    real p_av = .5*(p_cell + p_oo);
    real u_av = .5*(u_cell + u_oo);
    real v_av = .5*(v_cell + v_oo);
    ArtCompressibleRoe2D::compute(p_oo, u_oo, v_oo,
				  p_av, u_av, v_av,
				  p_cell, u_cell, v_cell,
				  nx, ny, beta_q,
				  flux_p, flux_u, flux_v);
    res_p[to] += flux_p;
    res_u[to] += flux_u;
    res_v[to] += flux_v;
  };
};

#endif













