// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Jul 14 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ArtCompressibleParabolicBSplitFlux2D_HH
#define ArtCompressibleParabolicBSplitFlux2D_HH

#include <mouse_config.h>

class ArtCompressibleParabolicBSplitFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <IncompressibleOperator2D.hh>
#include <ArtCompressibleRoe2D.hh>
#include <geometry2D.hh>

/**
*/
class ArtCompressibleParabolicBSplitFlux2D : public IncompressibleOperator2D
{
protected:
  /** */ real x1;
  /** */ real x2;
  /** */ real y1;
  /** */ real y2;
  /** */ real vx;
  /** */ real vy;
  /** */ real vmax;
  /** */ VarContainer::var1_t dp_dx;
  /** */ VarContainer::var1_t dp_dy;

public:
  /** */ ArtCompressibleParabolicBSplitFlux2D (carg_t arg);
  /** */ virtual ~ArtCompressibleParabolicBSplitFlux2D ();
  /** */ void comp_values(size_t i, real nx, real ny, 
			  real &p_out, real &u_out, real &v_out,
			  real &p_cell, real &u_cell, real &v_cell);
  /** */ void comp_fluxes(size_t i, real nx, real ny);
  /** */ void comp_simple_fluxes(size_t i, real nx, real ny);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void ArtCompressibleParabolicBSplitFlux2D::comp_values
(size_t i, real nx, real ny, 
 real &p_out, real &u_out, real &v_out,
 real &p_cell, real &u_cell, real &v_cell)
{
  // compute normal velocity
  real v_n = u[i]*nx + v[i]*ny;
  
  if (v_n >=0 ) {

    // inflow:
    // -------
    //   - take pressure from inside
    //   - take velocities from outside
    
    // set outside pressure (take value from inside)
    p_out = p[i];

    // compute outside velocities (parabolic profile)
    real x = PL->x(i);
    real y = PL->y(i);
    real xn = (x-x1)*(-vy) + (y-y1)*(vx);
    xn *= 2.0/DIST(x1,y1,x2,y2);
    real vabs = vmax*(1-SQR(xn-1));
    u_out = vabs*vx;
    v_out = vabs*vy;

    // debug!!
    u_out = u_oo;
    v_out = v_oo;

    // set cell values (...???)
    p_cell = p_out;
    u_cell = u_out;
    v_cell = v_out;
    
  } else {

    // outflow:
    // --------
    //   - take pressure from outside
    //   - take velocities from inside

    // set outside pressure (take free stream value)
    p_out = p_oo;

    // set outside velocities to cell values
    u_out = u[i];
    v_out = v[i];

    // set cell values (...???)
    p_cell = p[i];
    u_cell = u_out;
    v_cell = v_out;
  };
};

inline void ArtCompressibleParabolicBSplitFlux2D::comp_fluxes(size_t i, real nx, real ny)
{
  real flux_p, flux_u, flux_v;
  real p_out, u_out, v_out;
  real p_cell, u_cell, v_cell;
  comp_values(i, nx, ny, p_out, u_out, v_out, p_cell, u_cell, v_cell);
  real p_av = .5*(p_cell + p_out);
  real u_av = .5*(u_cell + u_out);
  real v_av = .5*(v_cell + v_out);
  ArtCompressibleRoe2D::compute(p_out, u_out, v_out, p_av, u_av, v_av, 
				p_cell, u_cell, v_cell,
				nx, ny, beta_q, flux_p, flux_u, flux_v);
  res_p[i] += flux_p;
  res_u[i] += flux_u;
  res_v[i] += flux_v;
};

inline void ArtCompressibleParabolicBSplitFlux2D::comp_simple_fluxes
(size_t i, real nx, real ny)
{
  real pp, uu, vv;
  
  // compute normal velocity
  real v_n = u[i]*nx + v[i]*ny;
  
  //if (v_n >=0 ) {
  if (true) { //debug version (forced inflow!!!)

    // inflow:
    // -------
    //   - take pressure from inside
    //   - take velocities from outside

    // set pressure (take value from inside)??
    pp = p[i];
    //pp = p_oo;

    // extrapolate pressure using the cell gradient
    //real d = sqrt(PL->Vol(i));
    //pp += d*(nx*dp_dx[i] + ny*dp_dy[i]);

    // compute velocities (parabolic profile)
    real x = PL->x(i);
    real y = PL->y(i);
    real xn = (x-x1)*(-vy) + (y-y1)*(vx);
    xn *= 2.0/DIST(x1,y1,x2,y2);
    real vabs = vmax*(1-SQR(xn-1));
    uu = vabs*vx;
    vv = vabs*vy;

    // debug!!
    //uu = u[i];
    //vv = v[i];
    //pp = p_oo;

  } else {

    // outflow:
    // --------
    //   - take pressure from outside
    //   - take velocities from inside

    // set pressure (take free stream value)
    pp = p_oo;
    //pp = p[i];

    // set velocities to cell values
    uu = u[i];
    vv = v[i];

    // debug!!
    //uu = u[i];
    //vv = v[i];
    //pp = 0;
  };
  
  // compute fluxes 
  {
    // compute convection speed [uc,vc]
    real uc = uu;//u[i];
    real vc = vv;//v[i];
    
    // add fluxes to residuals
    res_p[i] += (uc*beta_q)*nx + (vc*beta_q)*ny;
    res_u[i] += (uc*uu+pp)*nx + (uc*vv)*ny;
    res_v[i] += (vc*uu)*nx + (vc*vv+pp)*ny;
  };
};

inline void ArtCompressibleParabolicBSplitFlux2D::operate(size_t ie, size_t ip, 
							  size_t from, size_t to, 
							  real nx, real ny, 
							  bool mod_from, bool mod_to)
{
  if (mod_from) comp_simple_fluxes(from, nx, ny);
  if (mod_to) comp_simple_fluxes(to, nx, ny);
  //if (mod_from) comp_fluxes(from, nx, ny);
  //if (mod_to) comp_fluxes(to, nx, ny);
};


#endif








