// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Jul 13 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ArtCompressibleRoe2D_HH
#define ArtCompressibleRoe2D_HH

#include <mouse_config.h>

class ArtCompressibleRoe2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <IncompressibleFFlux2D.hh>

/**
*/
class ArtCompressibleRoe2D : public IncompressibleFFlux2D
{
protected:

public:
  /** */ ArtCompressibleRoe2D (carg_t arg);
  /** */ virtual ~ArtCompressibleRoe2D ();
  /** */ static void compute(real p_f, real u_f, real v_f,
			     real p_h, real u_h, real v_h,
			     real p_t, real u_t, real v_t,
			     real nx, real ny, real beta_q,
			     real &flux_p, real &flux_u, real &flux_v);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void ArtCompressibleRoe2D::compute(real p_f, real u_f, real v_f,
					  real p_h, real u_h, real v_h,
					  real p_t, real u_t, real v_t,
					  real nx, real ny, real beta_q,
					  real &flux_p, real &flux_u, real &flux_v)
{
  // compute normed interface-normal vector
  real geoksi = (nx * nx + (-ny) * (-ny));

  // mass and impulse transport through the cellwall
  real ppp = u_f * nx - v_f * (-ny);
  real ppm = u_t * nx - v_t * (-ny);
  real uup = u_f * ppp + nx * p_f;
  real uum = u_t * ppm + nx * p_t;
  real vvp = v_f * ppp - (-ny) * p_f;
  real vvm = v_t * ppm - (-ny) * p_t;
  
  // total fluxes
  real uuh = (u_h * nx - v_h * (-ny));
  real ss = sqrt(uuh * uuh + geoksi * beta_q);
  real dp = (p_t - p_f);
  real du = (u_t - u_f);
  real dv = (v_t - v_f);

  if (uuh < 0.0) {
    real dw2 = (dp * (ss - uuh) + beta_q * (nx * du - (-ny) * dv)) / (2.0 * ss);
    real ap1 = (uuh + ss);
    real ap2 = (u_h * uuh + u_h * ss + beta_q * nx) * (uuh + ss) / (beta_q * ss);
    real ap3 = (v_h * uuh + v_h * ss - beta_q * (-ny)) * (uuh + ss) / (beta_q * ss);
    flux_p = ppm * beta_q - ap1 * dw2;
    flux_u = uum - ap2 * dw2;
    flux_v = vvm - ap3 * dw2;
  }  else  {
    real dw3 = (dp * (ss + uuh) - beta_q * (nx * du - (-ny) * dv)) / (2.0 * ss);
    real am1 = (uuh - ss);
    real am2 = -(u_h * uuh - u_h * ss + beta_q * nx) * (uuh - ss) / (beta_q * ss); 
    real am3 = -(v_h * uuh - v_h * ss - beta_q * (-ny)) * (uuh - ss) / (beta_q * ss); 
    flux_p = ppp * beta_q + am1 * dw3;
    flux_u = uup + am2 * dw3;
    flux_v = vvp + am3 * dw3;
  };
};

inline void ArtCompressibleRoe2D::operate(size_t ie, size_t ip, size_t from, size_t to, 
					  real nx, real ny, bool mod_from, bool mod_to)
{
  real flux_p, flux_u, flux_v;
  compute(p_from[ie], u_from[ie], v_from[ie],
	  p_av[ie], u_av[ie], v_av[ie],
	  p_to[ie], u_to[ie], v_to[ie],
	  nx, ny, beta_q, flux_p, flux_u, flux_v);

  //.. Add to pointwise residual
  //.... .._to-side variables
  if (mod_to) {
    res_p[to] += flux_p;
    res_u[to] += flux_u;
    res_v[to] += flux_v;
  };
  //.... .._from-side variables
  if (mod_from) {
    res_p[from] -= flux_p;
    res_u[from] -= flux_u;
    res_v[from] -= flux_v;
  };
};

#endif









