// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Jul 14 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ArtCompressibleViscFlux2D_HH
#define ArtCompressibleViscFlux2D_HH

#include <mouse_config.h>

class ArtCompressibleViscFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <IncompressibleFFlux2D.hh>

/**
*/
class ArtCompressibleViscFlux2D : public IncompressibleFFlux2D
{
protected:
  /** */ real Re_n_inv;
  /** */ VarContainer::var1_t du_dx;
  /** */ VarContainer::var1_t du_dy;
  /** */ VarContainer::var1_t dv_dx;
  /** */ VarContainer::var1_t dv_dy;

public:
  /** */ ArtCompressibleViscFlux2D (carg_t arg);
  /** */ virtual ~ArtCompressibleViscFlux2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void ArtCompressibleViscFlux2D::operate(size_t ie, size_t ip, size_t from, size_t to, 
					       real nx, real ny, bool mod_from, bool mod_to)
{
  // average derivative on cell interface
  real u_x = 0.5 * (du_dx[from] + du_dx[to]);
  real u_y = 0.5 * (du_dy[from] + du_dy[to]);
  real v_x = 0.5 * (dv_dx[from] + dv_dx[to]);
  real v_y = 0.5 * (dv_dy[from] + dv_dy[to]);
  
  // The viscous fluxes for the cons. quantities
  real flux_u = Re_n_inv * (u_x * nx + u_y * ny);
  real flux_v = Re_n_inv * (v_x * nx + v_y * ny);
  
  // Add to raw residual (negative !)
  if (mod_to) {
    res_u[to] -= flux_u;
    res_v[to] -= flux_v;
  };
  if (mod_from) {
    res_u[from] += flux_u;
    res_v[from] += flux_v;
  };
};


#endif
