// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(COMMUNICATOR_HH)
#define COMMUNICATOR_HH

#include <string>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fcntl.h> // linux stuff ??
#include <strstream.h>

class Communicator;

#include "MObject.hh"
#include "EventChannel.hh"

/**
   Communicator.
*/
class Communicator : public MObject
{
  friend class EventChannel;

  /** */ TList<MObject*> *trash_list;

protected:
  /** */ EventChannel *first_channel;
  /** */ int server_socket;
  /** */ string host;
  /** */ unsigned short int port_number;

public:
  /** */ int argc;
  /** */ char **argv; 

  /** */ Communicator();
  /** */ ~Communicator();

  /** */ void SetupServer(unsigned short int port);
  /** */ void AddEventChannel(EventChannel *new_channel);
  /** */ void DeleteEventChannel(EventChannel *old_channel);
  /** */ void AddClientConnection(string host, unsigned short int port);
  /** */ void DeleteClientConnection(string host, unsigned short int port);
  /** */ void Incoming();
  /** */ void HandleRemoteEvents();

  /** */ virtual void HandleEvent(Event &E);
  /** */ void read_from_sock(int sock, void *buf, int n);
  /** */ void WriteInt(int, long int);
  /** */ long int ReadInt(int);
  /** */ void WriteReal(int, real);
  /** */ real ReadReal(int);
  /** */ void WriteString(int, string);
  /** */ string ReadString(int);

  /** */ void AddToTrash(MObject *mobj);
  /** */ void DelFromTrash(MObject *mobj);
  /** */ void CollectGarbage();
};

#endif







