// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 26 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef CompressibleCTWall2D_HH
#define CompressibleCTWall2D_HH

#include <mouse_config.h>

class CompressibleCTWall2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleBPOperator2D.hh>
#include <geometry2D.hh>

/**
*/
class CompressibleCTWall2D : public CompressibleBPOperator2D
{
protected:

public:
  /** */ CompressibleCTWall2D (carg_t arg);
  /** */ virtual ~CompressibleCTWall2D ();
  /** */ void operate(size_t il, size_t i);
  /** */ int vari;
};

inline void CompressibleCTWall2D::operate(size_t il, size_t i)
{
  // state before the correction
  real rho1 = rho[i];
  real rhou1 = rhou[i];
  real rhov1 = rhov[i];
  real rhoE1 = rhoE[i];

  // tangential correction
  real rhouv = rhou[i] * BPL->Nx(i) + rhov[i] * BPL->Ny(i);
  rhou[i] = rhou[i] - rhouv * BPL->Nx(i);
  rhov[i] = rhov[i] - rhouv * BPL->Ny(i);

  // state after the correction
  real rho2 = rho[i];
  real rhou2 = rhou[i];
  real rhov2 = rhov[i];
  real rhoE2 = rhoE[i];

  switch (vari) {

  case 1:
    // constant pressure (correction of energy)
    rhoE[i] = (2.0*rho2*rhoE1*rho1-rho2*rhou1*rhou1-rho2*rhov1*rhov1
	       +rhou2*rhou2*rho1+rhov2*rhov2*rho1)/rho1/rho2/2.0;
    break;

  case 2:
    // constant pressure (correction of density)
    rho[i] == -(rhou2*rhou2+rhov2*rhov2)*rho1/(2.0*rhoE1*rho1-rhou1*rhou1-rhov1*rhov1-2.0*rho1*rhoE2);
    break;

  case 3:
    // constant enthalpy (correction of energy)
    rhoE[i] = (2.0*rho2*rho2*gamma*rhoE1*rho1-rho2*rho2*gamma*rhou1*rhou1-rho2*rho2*gamma*rhov1*rhov1
	       +rho2*rho2*rhou1*rhou1+rho2*rho2*rhov1*rhov1+rho1*rho1*gamma*rhou2*rhou2
	       +rho1*rho1*gamma*rhov2*rhov2-rho1*rho1*rhou2*rhou2-rho1*rho1*rhov2*rhov2)/(rho1*rho1)/rho2/gamma/2.0;
    break;

  default:
    Error("unknown variation of the twall boundary condition");
  };
};

#endif
