// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Sep 1 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef CompressibleFixPFlux2D_HH
#define CompressibleFixPFlux2D_HH

#include <mouse_config.h>

class CompressibleFixPFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleOperator2D.hh>

/**
*/
class CompressibleFixPFlux2D : public CompressibleOperator2D
{
protected:
  /** */ void compute_fluxes(size_t i, real nx, real ny);
public:
  /** */ CompressibleFixPFlux2D (carg_t arg);
  /** */ virtual ~CompressibleFixPFlux2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleFixPFlux2D::compute_fluxes(size_t i, real nx, real ny)
{
  // compute conservative properties (pressure from outside)
  real rho_h = rho[i];
  real rhou_h = rhou[i];
  real rhov_h = rhov[i];
  real irho_h = 1.0 / rho_h;
  real rho_q_u = rhou_h * rhou_h * irho_h;
  real rho_q_v = rhov_h * rhov_h * irho_h;
  real rho_uv = rhou_h * rhov_h * irho_h;
  real q_roh_uv = rhou[i] * rhou[i] + rhov[i] * rhov[i];
  real rhoE_h = p_oo * igamma1 + 0.5 * q_roh_uv / rho[i];
  real p = gamma1 * (rhoE_h - 0.5 * (rho_q_u + rho_q_v));
  real h = (rhoE_h + p) * irho_h;

  //.... compute (h*n)*da for the cell interface
  res_rho[i]  += rhou_h * nx + rhov_h * ny;
  res_rhou[i] += (rho_q_u + p) * nx + rho_uv * ny;
  res_rhov[i] += rho_uv * nx + (rho_q_v + p) * ny;
  res_rhoE[i] += h * (rhou_h * nx + rhov_h * ny);
};

inline void CompressibleFixPFlux2D::operate(size_t ie, size_t ip, 
					    size_t from, size_t to, 
					    real nx, real ny, 
					    bool mod_from, bool mod_to)
{
  if (mod_from) compute_fluxes(from, nx, ny);
  if (mod_to) compute_fluxes(to, nx, ny);
};

#endif
