// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleInOutCh2D_HH
#define CompressibleInOutCh2D_HH

class CompressibleInOutCh2D;

#include <CompressibleBPOperator2D.hh>
#include <Parameter.hh>

/** */ 
class CompressibleInOutCh2D : public CompressibleBPOperator2D
{
public:
  /** */ CompressibleInOutCh2D(carg_t arg) : CompressibleBPOperator2D(arg) {
    AddClassName();
  };
  /** */ void operate(size_t il, size_t i);
};


inline void CompressibleInOutCh2D::operate(size_t il, size_t i)
{
  //.. Compute normal Mach number to decide
  real uv_n = (rhou[i] * BPL->Nx(i) + rhov[i] * BPL->Ny(i)) / rho[i];
  real q_roh_uv = rhou[i] * rhou[i] + rhov[i] * rhov[i];
  real a_speed = ((rhoE[i] * rho[i] -0.5 * q_roh_uv)
		  *gamma2) / (rho[i] * rho[i]);
  if(a_speed < 0.1)
    a_speed=0.1;    // avoid division by 0 if critical
  a_speed = sqrt(a_speed);
  real mach = uv_n / a_speed;
  //.. Decide upon Mach number
  //.... supersonic outflow, do nothing
  //cout << mach << " " << i << "   " << rho[i] << "," << rhou[i] << "," << rhov[i] << "," << rhoE[i] << endl;
  if (mach <= -1) {}
  //.... supersonic inflow, take everything
  else if (mach >= 1) {
    rho[i] = rho_oo;
    rhou[i] = rhou_oo;
    rhov[i] = rhov_oo;
    rhoE[i] = rhoE_oo;
  }
  //.... subsonic outflow, use pressure from outside to adjust energy
  else if (mach < 0) {
    rhoE[i] = p_oo * igamma1 + 0.5 * q_roh_uv / rho[i];
  }
  //.... subsonic inflow, take all from outside apart pressure
  //     stability problems -> keep Energy from inside
  else if (mach >= 0) {
    rho[i] = rho_oo;
    rhou[i] = rhou_oo;
    rhov[i] = rhov_oo;
  }
  //.... Error condition for corrupted mach number
  else {
    Error("corrupted Mach number");
  };
};


#endif




