// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 5 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef CompressibleInOutFlux2D_HH
#define CompressibleInOutFlux2D_HH

#include <mouse_config.h>

class CompressibleInOutFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleOperator2D.hh>

/**
*/
class CompressibleInOutFlux2D : public CompressibleOperator2D
{
protected:

public:
  /** */ CompressibleInOutFlux2D (carg_t arg);
  /** */ virtual ~CompressibleInOutFlux2D ();
  /** */ void EvalHalf(size_t i, real nx, real ny);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleInOutFlux2D::EvalHalf(size_t i, real nx, real ny)
{
  //.. Compute normal Mach number to decide
  real uv_n = (rhou[i] * nx + rhov[i] * ny) / rho[i];
  real q_roh_uv = rhou[i] * rhou[i] + rhov[i] * rhov[i];
  real a_speed = ((rhoE[i] * rho[i] -0.5 * q_roh_uv)
		  *gamma2) / (rho[i] * rho[i]);
  if(a_speed < 0.1) a_speed=0.1; // avoid division by 0 if critical
  a_speed = sqrt(a_speed);
  real mach = uv_n / a_speed;

  // the variables later used for the flux computation
  real rho_h, rhou_h, rhov_h, rhoE_h;

  // supersonic outflow
  // all values from within the cell
  if (mach <= -1) {
    rho_h = rho[i];
    rhou_h = rhou[i];
    rhov_h = rhov[i];
    rhoE_h = rhoE[i];
  }
  
  // supersonic inflow
  // all values prescribed by the far field conditions
  else if (mach >= 1) {
    rho_h = rho_oo;
    rhou_h = rhou_oo;
    rhov_h = rhov_oo;
    rhoE_h = rhoE_oo;
  }

  // subsonic outflow
  // use pressure from outside to adjust energy
  else if (mach < 0) {
    rho_h = rho[i];
    rhou_h = rhou[i];
    rhov_h = rhov[i];
    rhoE_h = p_oo * igamma1 + 0.5 * q_roh_uv / rho[i];
  }  

  // subsonic inflow
  // take all from outside apart pressure
  // stability problems -> keep Energy from inside
  else if (mach >= 0) {
    rho_h = rho_oo;
    rhou_h = rhou_oo;
    rhov_h = rhov_oo;
    rhoE_h = rhoE[i];
  };

  // some usefull quantities
  real irho_h = 1.0 / rho_h;
  real rho_q_u = rhou_h * rhou_h * irho_h;
  real rho_q_v = rhov_h * rhov_h * irho_h;
  real rho_uv = rhou_h * rhov_h * irho_h;
  real p = gamma1 * (rhoE_h - 0.5 * (rho_q_u + rho_q_v));
  real h = (rhoE_h + p) * irho_h;

  // compute (h*n)*da for the cell interface
  real flux_rho = rhou_h * nx + rhov_h * ny;
  real flux_rhou = (rho_q_u + p) * nx + rho_uv * ny;
  real flux_rhov = rho_uv * nx + (rho_q_v + p) * ny;
  real flux_rhoE = h * flux_rho;

  // add to the residual field
  res_rho[i] += flux_rho;
  res_rhou[i] += flux_rhou;
  res_rhov[i] += flux_rhov;
  res_rhoE[i] += flux_rhoE;
};

inline void CompressibleInOutFlux2D::operate(size_t ie, size_t ip, size_t from, size_t to, 
					     real nx, real ny, bool mod_from, bool mod_to)
{
  if (mod_from) EvalHalf(from, nx, ny);
  if (mod_to) EvalHalf(to, nx, ny);
};


#endif
