// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "CompressibleOperator2D.hh"
#include <math.h>

CompressibleOperator2D::CompressibleOperator2D(carg_t arg) : Operator(arg)
{
  AddClassName();
  PARA = ResolveVar<Parameter>(arg);
  Mach_oo = PARA->RealParam("Mach");
  alpha = PARA->RealParam("alpha");
  gamma = PARA->RealParam("gamma");

  // Compute some useful coefficients
  gamma1 = gamma - 1.0;
  gamma2 = gamma * (gamma - 1.0);
  igamma1 = 1.0 / (gamma - 1.0);

  // Compute the inflow conditions as cons. variables
  // (Norming system is p_oo and rho_oo)
  real alfabo = alpha * M_PI / 180.;
  real machx = Mach_oo*cos(alfabo);
  real machy = Mach_oo*sin(alfabo);
  real u_oo = machx * sqrt(gamma);
  real v_oo = machy * sqrt(gamma);
  p_oo = 1.;
  rho_oo = 1.;
  rhou_oo = u_oo*rho_oo;
  rhov_oo = v_oo*rho_oo;
  rhoE_oo = (1./(gamma-1.)+0.5*(u_oo*u_oo+v_oo*v_oo))*rho_oo;

  // get the field variables
  if (VL == NULL) {
    Error("I did not find a VarList where I can find (rho,rhou,rhov,rhoE).");
  };
  new_field_name = arg.P->ReadString();
  res_field_name = arg.P->ReadString();
  rho = VL->Get1DVar(new_field_name, "rho");
  rhou = VL->Get1DVar(new_field_name, "rhou");
  rhov = VL->Get1DVar(new_field_name, "rhov");
  rhoE = VL->Get1DVar(new_field_name, "rhoE");
  res_rho = VL->Get1DVar(res_field_name, "rho");
  res_rhou = VL->Get1DVar(res_field_name, "rhou");
  res_rhov = VL->Get1DVar(res_field_name, "rhov");
  res_rhoE = VL->Get1DVar(res_field_name, "rhoE");
};
 











