// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Sep 1 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef CompressibleParabolicBSplitFlux2D_HH
#define CompressibleParabolicBSplitFlux2D_HH

#include <mouse_config.h>

class CompressibleParabolicBSplitFlux2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CompressibleOperator2D.hh>
#include <geometry2D.hh>

/**
*/
class CompressibleParabolicBSplitFlux2D : public CompressibleOperator2D
{
protected:
  /** */ real x1;
  /** */ real x2;
  /** */ real y1;
  /** */ real y2;
  /** */ real vx;
  /** */ real vy;
  /** */ real vmax;
  /** */ inline void compute_fluxes(size_t i, real nx, real ny);

public:
  /** */ CompressibleParabolicBSplitFlux2D (carg_t arg);
  /** */ virtual ~CompressibleParabolicBSplitFlux2D ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleParabolicBSplitFlux2D::compute_fluxes(size_t i, real nx, real ny)
{
  // compute outside velocities (parabolic profile)
  real x = PL->x(i);
  real y = PL->y(i);
  real xn = (x-x1)*(-vy) + (y-y1)*(vx);
  xn *= 2.0/DIST(x1,y1,x2,y2);
  real vabs = vmax*(1-SQR(xn-1));

  //.. for the triangles adjacent to node "..from"
  //.... average on adjacent half edge, side "..from"
  real rho_h = rho[i];
  real rhou_h = rho[i]*vabs*vx;
  real rhov_h = rho[i]*vabs*vy;
  real rhoE_h = rhoE[i];
  //.... some usefull quantities
  real irho_h = 1.0 / rho_h;
  real rho_q_u = rhou_h * rhou_h * irho_h;
  real rho_q_v = rhov_h * rhov_h * irho_h;
  real rho_uv = rhou_h * rhov_h * irho_h;
  real p = gamma1 * (rhoE_h - 0.5 * (rho_q_u + rho_q_v));
  real h = (rhoE_h + p) * irho_h;

  //.... compute (h*n)*da for the cell interface
  res_rho[i]  += rhou_h * nx + rhov_h * ny;
  res_rhou[i] += (rho_q_u + p) * nx + rho_uv * ny;
  res_rhov[i] += rho_uv * nx + (rho_q_v + p) * ny;
  res_rhoE[i] += h * (rhou_h * nx + rhov_h * ny);
};

inline void CompressibleParabolicBSplitFlux2D::operate(size_t ie, size_t ip, 
						       size_t from, size_t to, 
						       real nx, real ny, 
						       bool mod_from, bool mod_to)
{
  if (mod_from) compute_fluxes(from, nx, ny);
  if (mod_to) compute_fluxes(to, nx, ny);
};

#endif
