// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CompressibleViscFlux2D_HH
#define CompressibleViscFlux2D_HH

class CompressibleViscFlux2D;

#include <CompressibleNavStokesOperator2D.hh>

class CompressibleViscFlux2D : public CompressibleNavStokesOperator2D
{
protected:
  /** */ VarContainer::var1_t drho_dx;
  /** */ VarContainer::var1_t drhou_dx;
  /** */ VarContainer::var1_t drhov_dx;
  /** */ VarContainer::var1_t drhoE_dx;
  /** */ VarContainer::var1_t drho_dy;
  /** */ VarContainer::var1_t drhou_dy;
  /** */ VarContainer::var1_t drhov_dy;
  /** */ VarContainer::var1_t drhoE_dy;

public:
  /** */ CompressibleViscFlux2D(carg_t arg);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void CompressibleViscFlux2D::operate(size_t ie, size_t ip, 
					    size_t from, size_t to, 
					    real nx, real ny, 
					    bool mod_from, bool mod_to)
{
  real d_u_dx, d_v_dx, d_T_dx, d_u_dy, d_v_dy, d_T_dy;
  real tau_xx, tau_xy, tau_yy, q_dot_x, q_dot_y;
  real flux_rhou, flux_rhov, flux_rhoE;

  real rho_h = rho_av[ie];
  real rhou_h = rhou_av[ie];
  real rhov_h = rhov_av[ie];
  real rhoE_h = rhoE_av[ie];

  // Average derivatives on cell interface
  real drho_dx_av = .5*(drho_dx[from]+drho_dx[to]);
  real drhou_dx_av = .5*(drhou_dx[from]+drhou_dx[to]);
  real drhov_dx_av = .5*(drhov_dx[from]+drhov_dx[to]);
  real drhoE_dx_av = .5*(drhoE_dx[from]+drhoE_dx[to]);
  real drho_dy_av = .5*(drho_dy[from]+drho_dy[to]);
  real drhou_dy_av = .5*(drhou_dy[from]+drhou_dy[to]);
  real drhov_dy_av = .5*(drhov_dy[from]+drhov_dy[to]);
  real drhoE_dy_av = .5*(drhoE_dy[from]+drhoE_dy[to]);

  //.. Compute primitive gradients d(u)/d(.), d(v)/d(.), d(T)/d(.)
  d_u_dx = (drhou_dx_av - drho_dx_av*rhou_h/rho_h)/rho_h;
  d_v_dx = (drhov_dx_av - drho_dx_av*rhov_h/rho_h)/rho_h;
  d_T_dx = gamma1 / (rho_h * rho_h) *
    (drhoE_dx_av * rho_h - drho_dx_av * rhoE_h
     - rhou_h * drhou_dx_av - rhov_h * drhov_dx_av
     + drho_dx_av / rho_h * (rhou_h * rhou_h + rhov_h * rhov_h));
  d_u_dy = (drhou_dy_av - drho_dy_av*rhou_h/rho_h)/rho_h;
  d_v_dy = (drhov_dy_av - drho_dy_av*rhov_h/rho_h)/rho_h;
  d_T_dy = gamma1 / (rho_h * rho_h) *
    (drhoE_dy_av * rho_h - drho_dy_av * rhoE_h
     - rhou_h * drhou_dy_av - rhov_h * drhov_dy_av
     + drho_dy_av / rho_h * (rhou_h * rhou_h + rhov_h * rhov_h));
  
  //.. Compute the fluxes out of that
  //.... tensions
  tau_xx = Fkenn2 * (2. * d_u_dx       -d_v_dy);
  tau_yy = Fkenn2 * (    -d_u_dx + 2. * d_v_dy);
  tau_xy = Fkenn3 * ( d_u_dy + d_v_dx);
  //.... heat flux
  q_dot_x = Fkenn4 * d_T_dx;
  q_dot_y = Fkenn4 * d_T_dy;
  
  //.. The viscous fluxes for the cons. quantities
  flux_rhou = tau_xx * nx + tau_xy * ny;
  flux_rhov = tau_xy * nx + tau_yy * ny;
  flux_rhoE =
    ((tau_xx * rhou_h + tau_xy * rhov_h) / rho_h + q_dot_x) * nx
    + ((tau_xy * rhou_h + tau_yy * rhov_h) / rho_h + q_dot_y) * ny;
  
  //.. Add to raw residual (negative !)
//   cout << flux_rhou << endl;
//   cout << flux_rhov << endl;
//   cout << flux_rhoE << endl << endl;
  if (mod_to) {
    res_rhou[to] -= flux_rhou;
    res_rhov[to] -= flux_rhov;
    res_rhoE[to] -= flux_rhoE;
  };
  if (mod_from) {
    res_rhou[from] += flux_rhou;
    res_rhov[from] += flux_rhov;
    res_rhoE[from] += flux_rhoE;
  };
};

#endif


