// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "DrawPolygon.hh"
#include <geometry2D.hh>

DrawPolygon::DrawPolygon(ConstructArg arg) : Mesh2DGeometryAction(arg)
{
  AddClassName();
  X = BuildObject<MRealList>(arg);
  Y = BuildObject<MRealList>(arg);
  C_POINT = BuildObject<MIntList>(arg);
  C_EDGE = BuildObject<MIntList>(arg);
  L = arg.P->ReadReal();
};

void DrawPolygon::Act()
{
  size_t j=0, p0=0, p1=0, p2=0;
  uint N;
  real x0, y0, x1, y1;
  real l_total, dx, dy;
  p1 = PL->AddEntry();
  MESH->FixPoint(p1);
  MESH->SetMatrixComp(p1, 0, 0, 0);
  MESH->SetNetDensity(p1, 0);
  PL->x(p1) = X->At(0);
  PL->y(p1) = Y->At(0);
  PL->code(p1) = C_POINT->At(0);
  p0 = p1;
  FORALL(i, X->) {
    x0 = X->At(i);
    y0 = Y->At(i);
    if (i == X->NumEntries() - 1) {
      x1 = X->At(0);
      y1 = Y->At(0);
    } else {
      x1 = X->At(i+1);
      y1 = Y->At(i+1);
    };
    l_total = DIST(x0, y0, x1, y1);
    N = (int) (l_total / L) + 1;
//     cout << x0 << " " << y0 << " " << x1 << " " << y1 << " " 
// 	 << N << " " << l_total << endl;
    dx = (x1 - x0) / N;
    dy = (y1 - y0) / N;
    for (j = 0; j < N; j++) {
      if ((i != X->NumEntries() - 1) || (j != N-1)) {
	p2 = PL->AddEntry();
	MESH->FixPoint(p2);
	MESH->SetMatrixComp(p2, 0, 0, 0);
	MESH->SetNetDensity(p2, 0);
	if (j != N-1) {
	  PL->x(p2) = x0 + (j+1) * dx;
	  PL->y(p2) = y0 + (j+1) * dy;
	  PL->code(p2) = C_EDGE->At(i);
	} else {
	  PL->x(p2) = X->At(i+1);
	  PL->y(p2) = Y->At(i+1);
	  PL->code(p2) = C_POINT->At(i+1);
	};
	TRI->AddFront(p1, p2);
	p1 = p2;
      };
    };
  };
  TRI->AddFront(p2, p0);
};










