// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "ElementList.hh"
#include "ConstructArg.hh"
#include <stdarg.h>

//
//.. constructor
//
ElementList::ElementList (ConstructArg arg) : MeshDataList(arg)
{
  AddClassName();
  DefineAsUniqueOnMesh();
  PL = FindObject<PointList>(false, true);
  filename += ".elements";
  element = new TList<Element*>(this);
  bool read_elements = arg.P->ReadBool();
  if (read_elements) ReadElements ();
};

//
//.. ReadElements
//
void ElementList::ReadElements () {
  ifstream s(filename.c_str());
  if (s.fail()) Error("couldn't open '" + filename + "'");
  size_t num_elements;
  s >> num_elements;
  unsigned int i_node, num_nodes;
  //for (i_element = 0; i_element < num_elements - 1; i_element = AddEntry()) {
  size_t i_element = 0;
  while (i_element < num_elements - 1) {
    i_element = AddEntry();
    s >> num_nodes;
    element->At(i_element) = new Element(num_nodes);
    for (i_node = 0; i_node < num_nodes; i_node++) {
      s >> Node(i_element,i_node);
    };
  };
};

//
//.. AddElement
//
size_t ElementList::AddElement (unsigned int num_nodes, ...) {
  va_list arg;
  va_start (arg, num_nodes);
  size_t i_element = AddEntry ();
  element->At(i_element) = new Element(num_nodes);
  for (unsigned int i_node = 0; i_node < num_nodes; i_node++) {
    Node (i_element, i_node) = va_arg (arg, size_t);
  };
  return i_element;
};

//
//.. AddElement
//
size_t ElementList::AddElement (unsigned int num_nodes, size_t* node_array) {
  size_t i_element = AddEntry ();
  element->At(i_element) = new Element(num_nodes);
  for (unsigned int i_node = 0; i_node < num_nodes; i_node++) {
    Node (i_element, i_node) = node_array[i_node];
  };
  return i_element;
};

//
//.. WriteToFile
//
void ElementList::WriteToFile () {
  cout << "writing elements ..." << endl;
  ofstream s(filename.c_str());
  s << NumActiveEntries () << '\n';
  size_t i_element;
  unsigned int i_node;
  for (i_element = begin(); i_element < end(); i_element = next(i_element)) {
    s << NumNodes(i_element) << ' ';
    for (i_node = 0; i_node < NumNodes(i_element); i_node++) {
      s << Node(i_element, i_node) << ' ';
    };
    s << endl;
  };
  cout << "done." << endl;
};

//
//.. x_middle
//
real ElementList::x_middle(size_t i_element) {
  real x=0;
  unsigned int i_node;
  for (i_node = 0; i_node < NumNodes(i_element); i_node++) {   
    x += PL->x(Node(i_element, i_node));
  };
  return x / NumNodes(i_element);
};

//
//.. y_middle
//
real ElementList::y_middle(size_t i_element) {
  real y=0;
  unsigned int i_node;
  for (i_node = 0; i_node < NumNodes(i_element); i_node++) {   
    y += PL->y(Node(i_element, i_node));
  };
  return y / NumNodes(i_element);
};

//
//.. z_middle
//
real ElementList::z_middle(size_t i_element) {
  real z=0;
  unsigned int i_node;
  for (i_node = 0; i_node < NumNodes(i_element); i_node++) {   
    z += PL->x(Node(i_element, i_node));
  };
  return z / NumNodes(i_element);
};














