// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Thu Aug 26 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "EliminateEdges2D.hh"
#include <geometry2D.hh>

EliminateEdges2D::EliminateEdges2D(carg_t arg)
  : Mesh2DAction(arg)
{
  AddClassName();
  l_min = arg.P->ReadReal();
  EL = FindObject<ElementList>(false, true);
  PL = FindObject<PointList>(false, true);
};

EliminateEdges2D::~EliminateEdges2D()
{
};

void EliminateEdges2D::Act()
{
  real l1, l2, x0, y0, a11, a22, a12, nd;
  size_t i, p1, p2, e1, e2;
  size_t p11, p12, p13, p21, p22, p23;
  real x11, x12, x13, x21, x22, x23;
  real y11, y12, y13, y21, y22, y23;
  int j;
  FORALL(i, EL->) {//if (MESH->ElementMarked(i)) {
    for (j = 0; j < EL->NumNodes(i); j++) {
      p1 = EL->PermNode(i, j);
      p2 = EL->PermNode(i, j + 1);
      if ((MESH->NumQuads(p1) == 0) && (MESH->NumQuads(p2) == 0)) {
	bool e1_found, e2_found;
	MESH->FindElements(p1, p2, e1, e2, e1_found, e2_found);
	if (e1_found && e2_found) {
	  // modification needed for quads @@
	  if ((EL->NumNodes(e1) == 3) && (EL->NumNodes(e2) == 3)) {
	    p11 = EL->Node(e1, 0);
	    p12 = EL->Node(e1, 1);
	    p13 = EL->Node(e1, 2);
	    p21 = EL->Node(e2, 0);
	    p22 = EL->Node(e2, 1);
	    p23 = EL->Node(e2, 2);
	    x11 = PL->x(p11);
	    x12 = PL->x(p12);
	    x13 = PL->x(p13);
	    x21 = PL->x(p21);
	    x22 = PL->x(p22);
	    x23 = PL->x(p23);
	    y11 = PL->y(p11);
	    y12 = PL->y(p12);
	    y13 = PL->y(p13);
	    y21 = PL->y(p21);
	    y22 = PL->y(p22);
	    y23 = PL->y(p23);
	    x0 = .5*(PL->x(p1) + PL->x(p2));
	    y0 = .5*(PL->y(p1) + PL->y(p2));
	    a11 = .25*(MESH->A11(p1) + MESH->A11(p2));
	    a12 = .25*(MESH->A12(p1) + MESH->A12(p2));
	    a22 = .25*(MESH->A22(p1) + MESH->A22(p2));
	    MESH->Transform(x0, y0, a11, a12, a22, x11, y11);
	    MESH->Transform(x0, y0, a11, a12, a22, x12, y12);
	    MESH->Transform(x0, y0, a11, a12, a22, x13, y13);
	    MESH->Transform(x0, y0, a11, a12, a22, x21, y21);
	    MESH->Transform(x0, y0, a11, a12, a22, x22, y22);
	    MESH->Transform(x0, y0, a11, a12, a22, x23, y23);
	    nd = .5 * (MESH->ND(p1) + MESH->ND(p2));
	    l1 = DIST(x11, y11, x12, y12) + DIST(x12, y12, x13, y13) + DIST(x13, y13, x11, y11);
	    l1 *= nd/3.0;
	    l2 = DIST(x21, y21, x22, y22) + DIST(x22, y22, x23, y23) + DIST(x23, y23, x21, y21);
	    l2 *= nd/3.0;
	    //cout << l1 << " " << l2 << endl;
	    if (0.5 * (l1 + l2) < l_min) {
	      if (MESH->CheckPoint(p1, p2)) {
		//cout << "deleting " << p1 << " --> " << p2 << endl;
		MESH->UnionPoints(p1, p2);
	      } else if (MESH->CheckPoint(p2, p1)) {
		//cout << "deleting " << p1 << " <-- " << p2 << endl;
		MESH->UnionPoints(p2, p1);
	      };
	      break;
	    };
	  };
	};
      };
    };
  };
};
