// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "Event.hh"
#include "MObject.hh"

//
//.. constructor
//
Event::Event ()
{
  invalid = true;
  done = false;
  single_recipient = false;
  broadcast = false;
  send_only_up = false;
  send_only_down = false;
  only_same_mesh = false;
  send_to_other_nodes = false;
  num_reactions = 0;
  name = "EmptyEvent";

  in_pointer = new TList<MObject*> (5, 5, NULL);
  in_real = new TList<real> (5, 5, 0);
  in_int = new TList<long int> (5, 5, 0);
  in_string = new TList<string> (5, 5, "");

  out_pointer = new TList<MObject*> (5, 5, NULL);
  out_real = new TList<real> (5, 5, 0);
  out_int = new TList<long int> (5, 5, 0);
  out_string = new TList<string> (5, 5, "");

  recipient_id = "";
  sender = NULL;
  sender_id = "";
  sender_mesh_id = "";
  origin_channel = NULL;

  max_dive_depth = -1;
  current_dive_depth = 0;
};

//
//.. destructor
//
Event::~Event ()
{
  delete in_pointer;
  delete in_real;
  delete in_int;
  delete in_string;
  delete out_pointer;
  delete out_real;
  delete out_int;
  delete out_string;
};

//
//.. React
//
void Event::React (MObject *obj)
{
  if (!broadcast) done = true;
  num_reactions++;
  // more to come ? ...
#ifdef __DEBUG
  {
    string text = "object of class '" + obj->TopClassName() + "' ";
    text = text + "reacted on '" + name + "' event";
    obj->Mesg(text);
  }
#endif
};

//
//.. MarkAsDone
//
void Event::MarkAsDone ()
{
  done = true;
  // more to come ...
};

//
//.. Errror
//
void Event::Error ()
{
  sender->Error("an error occured with the following event : " + name);
};

//
//.. InPointer
//
MObject* Event::InPointer (size_t i)
{
  if (i >= in_pointer->NumEntries()) {
    sender->Error("trying to access non existing inputdata from event : " + name);
  };
  return in_pointer->At(i); 
};

//
//.. InInt
//
long int Event::InInt (size_t i)
{
  if (i >= in_int->NumEntries()) {
    sender->Error("trying to access non existing inputdata from event : " + name);
  };
  return in_int->At(i); 
};

//
//.. InReal
//
real Event::InReal (size_t i)
{
  if (i >= in_real->NumEntries()) {
    sender->Error("trying to access non existing inputdata from event : " + name);
  };
  return in_real->At(i); 
};

//
//.. InString
//
string Event::InString (size_t i)
{
  if (i >= in_string->NumEntries()) {
    sender->Error("trying to access non existing inputdata from event : " + name);
  };
  return in_string->At(i); 
};

//
//.. OutPointer
//
MObject* Event::OutPointer (size_t i)
{
  if (i >= out_pointer->NumEntries()) {
    sender->Error("trying to access non existing outputdata from event : " + name);
  };
  return out_pointer->At(i); 
};

//
//.. OutInt
//
long int Event::OutInt (size_t i)
{
  if (i >= out_int->NumEntries()) {
    sender->Error("trying to access non existing outputdata from event : " + name);
  };
  return out_int->At(i); 
};

//
//.. OutReal
//
real Event::OutReal (size_t i)
{
  if (i >= out_real->NumEntries()) {
    sender->Error("trying to access non existing outputdata from event : " + name);
  };
  return out_real->At(i); 
};

//
//.. OutString
//
string Event::OutString (size_t i)
{
  if (i >= out_string->NumEntries()) {
    sender->Error("trying to access non existing outputdata from event : " + name);
  };
  return out_string->At(i); 
};

//
//.. Print
//
void Event::Print()
{
  size_t i;
  cout << "Event:" << endl;
  cout << num_reactions << endl;
  cout << invalid << endl;
  cout << done << endl;
  cout << single_recipient << endl;
  cout << broadcast << endl;
  cout << send_to_other_nodes << endl;
  cout << send_only_up << endl;
  cout << send_only_down << endl;
  cout << only_same_mesh << endl;
  cout << name << endl;
  cout << recipient_id << endl;
  cout << sender_id << endl;
  cout << sender_mesh_id << endl;
  
  cout << NumOutIndices() << endl;
  for (i = 0; i < NumOutIndices(); i++) cout << OutInt(i) << endl;
  cout << NumOutReals() << endl;
  for (i = 0; i < NumOutReals(); i++) cout << OutReal(i) << endl;
  cout << NumOutStrings() << endl;
  for (i = 0; i < NumOutStrings(); i++) cout << OutString(i) << endl;

  cout << NumInIndices() << endl;
  for (i = 0; i < NumInIndices(); i++) cout << InInt(i) << endl;
  cout << NumInReals() << endl;
  for (i = 0; i < NumInReals(); i++) cout << InReal(i) << endl;
  cout << NumInStrings() << endl;
  for (i = 0; i < NumInStrings(); i++) cout << InString(i) << endl << endl;
};









