// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(EVENT_HH)
#define EVENT_HH

#include <string>

class Event;
class MObject;
class Parser;

#include <TList.hh>

/**
   Event
*/
class Event 
{
  friend class MObject;
  friend class EventChannel;
  friend class Communicator;

private:
  /** */ MObject *carrier;

protected:
  /** */ bool done;

  /** */ TList<MObject*> *in_pointer;
  /** */ TList<real> *in_real;
  /** */ TList<long int> *in_int;
  /** */ TList<string> *in_string;

  /** */ TList<MObject*> *out_pointer;
  /** */ TList<real> *out_real;
  /** */ TList<long int> *out_int;
  /** */ TList<string> *out_string;

  /** */ int max_dive_depth;
  /** */ int current_dive_depth;

public:
  /** */ unsigned int num_reactions;
  /** */ bool invalid;
  /** */ bool single_recipient;
  /** */ bool broadcast;
  /** */ bool send_to_other_nodes;
  /** */ bool send_only_up;
  /** */ bool send_only_down;
  /** */ bool only_same_mesh;
  /** */ string name;
  /** */ Parser *P;
  /** */ string recipient_id;
  /** */ string sender_id;
  /** */ string sender_mesh_id;
  /** */ MObject *sender;
  /** */ EventChannel *origin_channel;

  /** */ Event ();
  /** */ ~Event ();

  /** */ void React (MObject *obj);
  /** */ void MarkAsDone();
  /** */ int Success () { return (num_reactions > 0); };
  /** */ void Error();
  /** */ bool Done() { return done; };

  /** */ void AddInPointer (MObject *a_in) { in_pointer->NewEntry() = a_in; };
  /** */ void AddInReal (real a_in) { in_real->NewEntry() = a_in; };
  /** */ void AddInInt (long int a_in) { in_int->NewEntry() = a_in; };
  /** */ void AddInString (string a_in) { in_string->NewEntry() = a_in; };

  /** */ void AddOutPointer (MObject *a_out) { out_pointer->NewEntry() = a_out; };
  /** */ void AddOutReal (real a_out) { out_real->NewEntry() = a_out; };
  /** */ void AddOutInt (long int a_out) { out_int->NewEntry() = a_out; };
  /** */ void AddOutString (string a_out) { out_string->NewEntry() = a_out; }; 

  /** */ MObject* InPointer (size_t i);
  /** */ long int InInt (size_t i);
  /** */ real InReal (size_t i);
  /** */ string InString (size_t i);

  /** */ MObject* OutPointer (size_t i);
  /** */ long int OutInt (size_t i);
  /** */ real OutReal (size_t i);
  /** */ string OutString (size_t i);

  /** */ size_t NumOutPointers() { return out_pointer->NumEntries(); }
  /** */ size_t NumOutIndices() { return out_int->NumEntries(); }
  /** */ size_t NumOutReals() { return out_real->NumEntries(); }
  /** */ size_t NumOutStrings() { return out_string->NumEntries(); }

  /** */ size_t NumInPointers() { return in_pointer->NumEntries(); }
  /** */ size_t NumInIndices() { return in_int->NumEntries(); }
  /** */ size_t NumInReals() { return in_real->NumEntries(); }
  /** */ size_t NumInStrings() { return in_string->NumEntries(); }

  /** */ void Print();
};


#endif
















