// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri Jun 18 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "IncompressibleOperator2D.hh"
#include <math.h>

IncompressibleOperator2D::IncompressibleOperator2D(carg_t arg)
  : Operator(arg)
{
  AddClassName();
  PARA = ResolveVar<Parameter>(arg);
  beta_q = PARA->RealParam("beta_q");  
  alpha = PARA->RealParam("alpha");

  // Compute the inflow conditions as cons. variables
  // (Norming system is p_oo and rho_oo)
  real alfabo = alpha * M_PI / 180.;
  u_oo = cos(alfabo);
  v_oo = sin(alfabo);
  p_oo = 1.;

  new_field_name = arg.P->ReadString();
  res_field_name = arg.P->ReadString();
  p = VL->Get1DVar(new_field_name, "p");
  u = VL->Get1DVar(new_field_name, "u");
  v = VL->Get1DVar(new_field_name, "v");
  res_p = VL->Get1DVar(res_field_name, "p");
  res_u = VL->Get1DVar(res_field_name, "u");
  res_v = VL->Get1DVar(res_field_name, "v");
};

IncompressibleOperator2D::~IncompressibleOperator2D()
{
};

