// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri Jun 18 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef IncompressibleOperator2D_HH
#define IncompressibleOperator2D_HH

#include <mouse_config.h>

class IncompressibleOperator2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>
#include <Parameter.hh>

/**
*/
class IncompressibleOperator2D : public Operator
{
protected:
  /** object holding some parameters */
  Parameter *PARA;

  /** angle of attack */ 
  real alpha;

  /** free stream x velocity */
  real u_oo;

  /** free stream y velocity */
  real v_oo;

  /** free stream pressure */
  real p_oo;

  /** artificial compressibility value */
  real beta_q;

  /** the name of the variable field used for computations */
  string new_field_name;

  /** the name of the residual field */
  string res_field_name;

  /** pressure field */
  VarContainer::var1_t p;

  /** x velocity field */
  VarContainer::var1_t u;

  /** y velocity field */
  VarContainer::var1_t v;

  /** pressure residual field */
  VarContainer::var1_t res_p;

  /** x velocity residual field */
  VarContainer::var1_t res_u;

  /** y velocity residual field */
  VarContainer::var1_t res_v;

public:
  /** */ IncompressibleOperator2D (carg_t arg);
  /** */ virtual ~IncompressibleOperator2D ();
};


#endif
