// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(ISOVALUE_HH)
#define ISOVALUE_HH

class IsoValue;

#include <MObject.hh>
#include <ConstructArg.hh>
#include <PointList.hh>

/**
   interface for values which shall be used for isoline plots.
*/
class IsoValue : public MObject
{
protected:
  /** */ string text;
  /** */ PointList *PL;
  /** */ TList<real> *plot_value;

public:
  /** */ 
  IsoValue(ConstructArg arg) : MObject(arg) {
    AddClassName();
    PL = FindObject<PointList>(false, true);
    plot_value = new TList<real>(PL);
    text = arg.P->ReadString();
  };

  /**
     get the iso value for a given point.
     @param p index of the point
     @return the plot value
  */
  real Value(size_t p) { return plot_value->At(p); };

  /**
     compute all plot values.
     This has to be overwritten for customized plots.
  */
  virtual void CalcValues() = 0;

  /**
     name of the value (e.g. Mach, pressure ...), which can be choosen
     in the isoline dialog.
     @return the name
  */
  string Text() { return text; };
};

#endif







