// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

/**********************************************************************

	--- Qt Architect generated file ---

	File: IsoplotConfigDlg.cc
	Last generated: Tue Aug 11 12:38:10 1998

 *********************************************************************/

#include <mouse_config.h>
#ifdef WITH_QT

#include "IsoplotConfigDlg.hh"
#include "IsoplotConfigDlgData.cc"


//
//.. constructor
//
IsoplotConfigDlg::IsoplotConfigDlg(MQIsolinePlotter *a_plotter,
				   QWidget* parent, 
				   const char* name)
  : IsoplotConfigDlgData( parent, name )
{
  plotter = a_plotter;
  int i;
  for (i = 0; i < plotter->NumIsos(); i++) {
    selector->insertItem(plotter->IsoV(i)->Text().c_str());
  };
  min_edit->setValidator(new QDoubleValidator(this));
  max_edit->setValidator(new QDoubleValidator(this));
  num_edit->setValidator(new QIntValidator(this));
  GetValues();
  GetFlags();
};

//
//.. destructor
//
IsoplotConfigDlg::~IsoplotConfigDlg()
{
};

//
//.. AcceptChanges
//
void IsoplotConfigDlg::AcceptChanges()
{
  QString min_s(min_edit->text());
  QString max_s(max_edit->text());
  QString num_s(num_edit->text());
  plotter->SetIso(selector->currentItem());
  plotter->CalcValues();
  if (radio_grey->isChecked()) plotter->color_type = 0;
  if (radio_redblue->isChecked()) plotter->color_type = 1;
  if (radio_rainbow->isChecked()) plotter->color_type = 2;
  plotter->SetLevels(min_s.toDouble(), max_s.toDouble(), num_s.toInt());
  plotter->SetLines(check_lines->isChecked());
  plotter->SetFill(check_fill->isChecked());
  plotter->colormap = check_map->isChecked();
  plotter->autolevels = check_auto->isChecked();
  accept();
};

//
//.. GetFlags
//
void IsoplotConfigDlg::GetFlags()
{
  switch(plotter->color_type) {
  case 0:
    radio_grey->setChecked(true);
    radio_redblue->setChecked(false);
    radio_rainbow->setChecked(false);
    break;
  case 1:
    radio_grey->setChecked(false);
    radio_redblue->setChecked(true);
    radio_rainbow->setChecked(false);
    break;
  case 2:
    radio_grey->setChecked(false);
    radio_redblue->setChecked(false);
    radio_rainbow->setChecked(true);
    break;
  };
  check_lines->setChecked(plotter->Lines());
  check_fill->setChecked(plotter->Fill());
  check_map->setChecked(plotter->colormap);
  check_auto->setChecked(plotter->autolevels);
};

//
//.. GetValues
//
void IsoplotConfigDlg::GetValues()
{
  QString s;
  s = s.setNum(plotter->Level(0));
  min_edit->setText(s);
  s = s.setNum(plotter->Level(plotter->NumLevels() - 1));
  max_edit->setText(s);
  s = s.setNum(plotter->NumLevels());
  num_edit->setText(s);
};

#endif







