// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Jun 7 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef LeastSquareInterpolator2D_HH
#define LeastSquareInterpolator2D_HH

#include <mouse_config.h>

class LeastSquareInterpolator2D;

#include <mouse_types.hh>
#include <TList.hh>
#include <SmallSquareMatrix.hh>

/**
*/
class LeastSquareInterpolator2D
{
protected:
  /** */ static const uint num_coeff = 5;

  /** */ typedef SmallSquareMatrix<real, num_coeff> matrix;
  /** */ typedef SmallVector<real, num_coeff> vector;

  /** */ TList<real> *X_list;
  /** */ TList<real> *Y_list;
  /** */ TList<real> *F;
  /** */ TList<real> *W;
  /** */ real x0;
  /** */ real y0;

  /** */ matrix M;
  /** */ matrix M_inv;
  /** */ vector COEFF;
  /** */ vector RHS;

  /** */ bool M_uptodate;
  /** */ bool RHS_uptodate;

  /** */ void ComputeMatrix();
  /** */ void ComputeRhs();
  /** */ void ComputeCoeff();
  /** */ real X(size_t i) { return X_list->At(i); };
  /** */ real Y(size_t i) { return Y_list->At(i); };

public:
  /** */ LeastSquareInterpolator2D();
  /** */ void Reset();
  /** */ size_t AddPoint(real x, real y);
  /** */ void SetValue(size_t i, real f);
  /** */ void SetWeight(size_t i, real w);
  /** */ void SetPosition(real x, real y);
  /** */ real ComputeValue(real x, real y);
  /** */ virtual ~LeastSquareInterpolator2D();
};


#endif
