// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQApplication.hh"
#include <qprogbar.h>
#include <qprogdlg.h>

//
//.. constructor
//
MQApplication::MQApplication(ConstructArg arg) : QObject(0, 0), Action(arg)
{
  AddClassName();
  INIT_ACTION = BuildObject<Action>(arg);
  RUN_ACTION = BuildObject<Action>(arg);
};

//
//.. Act
//
void MQApplication::Act()
{
  qapp = new QApplication(ancestor->argc, ancestor->argv);
  dlg = new MQApplicationDlg();
  qapp->setMainWidget(dlg);
  dlg->show();
  Menu("File")->insertItem("exit", qapp, SLOT(quit()));
  Menu("File")->insertSeparator();  
  connect(dlg->run_button, SIGNAL(clicked()), this ,SLOT(RunSteps()));
  INIT_ACTION->Act();
  qapp->exec();
};

//
//.. RunThread
//
void MQApplication::RunThread()
{
};
 
//
//.. SetSteps
//
void MQApplication::SetSteps(int steps)
{
  dlg->lcdbar1->setValue(steps/100000);
  dlg->lcd1->display(steps/100000);
  steps -= steps/100000*100000;
  dlg->lcdbar2->setValue(steps/10000);
  dlg->lcd2->display(steps/10000);
  steps -= steps/10000*10000;
  dlg->lcdbar3->setValue(steps/1000);
  dlg->lcd3->display(steps/1000);
  steps -= steps/1000*1000;
  dlg->lcdbar4->setValue(steps/100);
  dlg->lcd4->display(steps/100);
  steps -= steps/100*100;
  dlg->lcdbar5->setValue(steps/10);
  dlg->lcd5->display(steps/10);
  steps -= steps/10*10;
  dlg->lcdbar6->setValue(steps);
  dlg->lcd6->display(steps);
};

//
//.. SLOT:RunSteps
//
void MQApplication::RunSteps()
{
  int num_steps = 0;
  num_steps += 100000 * dlg->lcd1->intValue();
  num_steps += 10000 * dlg->lcd2->intValue();
  num_steps += 1000 * dlg->lcd3->intValue();
  num_steps += 100 * dlg->lcd4->intValue();
  num_steps += 10 * dlg->lcd5->intValue();
  num_steps += 1 * dlg->lcd6->intValue();
  {
    int i;
    QProgressDialog p("executing steps ...", "Stop", num_steps, 0, 0, true);
    for (i = 0; i < num_steps; i++) {
      p.setProgress( i );
      if (p.wasCancelled()) break;
      RUN_ACTION->Act();
    };
    SetSteps(num_steps - i);
  };
};

#endif







