// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(MQDRAWWIDGET_HH)
#define MQDRAWWIDGET_HH

#include <mouse_config.h>
#ifdef WITH_QT

class MQDrawWidget;

#include <MQWidget.hh>
#include <PointList.hh>
#include <ElementList.hh>
#include <qpainter.h>

/**
   provides some basic coordinate scaling and transformation methods.
*/
class MQDrawWidget : public MQWidget
{
  Q_OBJECT;

protected:
  /** */ QRect scale_rect;
  /** */ bool other_scale;
  /** */ real x_screen;
  /** */ real y_screen;
  /** */ real x_origin;
  /** */ real y_origin;
  /** */ virtual void resizeEvent(QResizeEvent *E);
  /** */ bool first_paint;
  /** */ bool is_painting;
  /** */ PointList *PL;
  /** */ ElementList *EL;

public:
  /** */
  MQDrawWidget(ConstructArg arg, QWidget *parent=0, const char *name=0, WFlags f=0);

  /** */ void WorldToScreen(real xw, real yw, int &xs, int &ys);
  /** */ void ScreenToWorld(int xs, int ys, real &xw, real &yw);
  /** */ void Zoom(real xz, real yz);
  /** */ void Scale(real xs, real ys);
  /** */ void Move(real dx, real dy);
  /** */ void Autoscale();
  /** */ void OptimizeAspectRatio();
  /** */ bool IsIn(real x, real y) { 
    return ((x>=x_origin) && (x<=x_origin+x_screen) 
	    && (y>=y_origin) && (y<=y_origin+y_screen));
  };
  /** */ real DiagonalLength() { return sqrt(x_screen*x_screen + y_screen*y_screen); };
  /** */ void DrawLine(QPainter *P, real x1, real y1, real x2, real y2);
  /** */ void DrawLine(QPainter *P, size_t p1, size_t p2);
  /** */ void DrawVector(QPainter *P, real x, real y, 
			 real vx, real vy, real size, real head_size);
  /** */ void DrawCircle(QPainter *P, real x, real y, real r);
};

#endif
#endif







