// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(MQIsolinePlotter_HH)
#define MQIsolinePlotter_HH

#include <mouse_config.h>
#ifdef WITH_QT

class MQIsolinePlotter;

#include <MQMeshPlotter.hh>
#include <qpointarray.h>
#include <IsoValue.hh>
#include <MQPlotWidget.hh>

/** */
struct BadReal_error{};

/** */
class MQIsolinePlotter : public MQMeshPlotter
{
  Q_OBJECT;

protected:
  /** */ IsoValue **ISO;
  /** */ uint num_isos;
  /** */ uint current_iso;
  /** */ uint num_levels;
  /** */ QColor **color;
  /** */ real *level;
  /** */ void ComputePoint(long int p0, long int p1, real v0, real v1, 
			   real iso, real &xiso, real &yiso);
  /** */ int CreateLine(real iso, size_t *p, real *v, real *x, real *y);
  /** */ void CheckReal(real a);

public:
  /** */ int color_type;
  /** */ bool lines;
  /** */ bool fill;
  /** */ bool colormap;
  /** */ bool autolevels;

  /** constructor. */
  MQIsolinePlotter(ConstructArg arg, QWidget *parent=0, const char *name=0);

  /** */ virtual void DrawColorMap(MQPlotWidget *pw, QPainter *P, const QRect *R);
  /** */ virtual void Plot(MQPlotWidget *pw, QPainter *P, const QRect *R);
  /** */ virtual QDialog* CreateConfigDlg(QWidget *parent);
  /** */ int NumIsos() { return num_isos; };
  /** */ IsoValue* IsoV(int i) { return ISO[i]; };
  /** */ void SetLevels(real minv, real maxv, int nl);
  /** */ void AutoLevels(int nl, MQPlotWidget *pw);
  /** */ real Level(int i) { return level[i]; };
  /** */ int NumLevels() { return num_levels; };
  /** */ void SetIso(int i) { current_iso = i; };
  /** */ void CalcValues() { ISO[current_iso]->CalcValues(); };
  /** */ bool Lines() { return lines; };
  /** */ bool Fill() { return fill; };
  /** */ void SetLines(bool b) { lines = b; };
  /** */ void SetFill(bool b) { fill = b; };
};

#endif
#endif







