// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQMultiGridPopPlotter.hh"
#include <ConstructArg.hh>

MQMultiGridPopPlotter::MQMultiGridPopPlotter(carg_t arg) : MQPopPlotter(arg)
{
  AddClassName();
  MultiGridAgent<PopList2D> *mga = LookForObject<MultiGridAgent<PopList2D> >(false, true);
  mg_agents = new TList<MultiGridAgent<PopList2D>*>(5,5,NULL);
  /*
  while (!arg.P->eof()) {
    mga = ResolveVar<MultiGridAgent<PopList2D> >(arg);
    mg_agents->NewEntry() = mga;
  };
  */
  int level = arg.P->ReadInt();
  for (int i = 0; i < level; i++) {
    if (mga == NULL) Error("requesting a non existent level");
    mg_agents->NewEntry() = mga;
    mga = mga->Next();
  };
};

size_t MQMultiGridPopPlotter::CoarsePoint(size_t p)
{
  FORALL(i, mg_agents->) {
    p = mg_agents->At(i)->CoarsePoint(p);
  };
  return p;
};

void MQMultiGridPopPlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  /*
  FORALL(i, PL->) {
    size_t p1 = i;
    real x1 = PL->x(p1);
    real y1 = PL->y(p1);
    for (size_t j = 0; j < POP->NumFLinks(i); j++) {
      size_t p2 = POP->FLinkPoint(i, j);
      if (pw->IsIn(.5*(PL->x(p1) + PL->x(p2)), .5*(PL->y(p1) + PL->y(p2)))) {
	if (CoarsePoint(p1) != CoarsePoint(p2)) {
	  PlotFVector(pw, P, x1, y1, i, j, p1, p2);
	};
      };
    };
  };
  */
  
  PointList* PPL;
  try {
    MultiGridAgent<PopList2D> *mga = mg_agents->At(mg_agents->last());
    PPL = mga->CoarsePL();
  } catch (NotFound_error) {
    PPL = PL;
  };

  FORALL(i, EL->) {
    if (ElementInside(pw, i, R)) {
      for (size_t j = 0; j < EL->NumNodes(i); j++) {
	size_t p1 = EL->PermNode(i, j);
	size_t p2 = EL->PermNode(i, j+1);
	size_t pc1 = CoarsePoint(p1);
	size_t pc2 = CoarsePoint(p2);
	if (pc1 != pc2) {
	  //P->setPen(QPen(black, 2*mg_agents->NumEntries()+1));
	  P->setPen(black);
	  PlotSegment(pw, P, i, p1, p2);
	  
	  // get the coordinates of the coarse point
	  real xr = PPL->x(pc1);
	  real yr = PPL->y(pc1);
	  int x, y;
	  pw->WorldToScreen(xr, yr, x, y);
	  P->drawLine(x-2, y-2, x+2, y+2);
	  P->drawLine(x-2, y+2, x+2, y-2);
	};
      };
    };
  };
  //PlotBVectors(pw, P, R);
};

#endif
