// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include <mouse_config.h>
#ifdef WITH_QT

#include "MQPopPlotter.hh"
#include <geometry2D.hh>

MQPopPlotter::MQPopPlotter(ConstructArg arg, QWidget *parent=0, const char *name=0) 
  : MQMeshPlotter(arg, parent, name)
{
  AddClassName();
  POP = FindObject<PopList2D>(false, true);
};

void MQPopPlotter::PlotFVector(MQPlotWidget *pw, QPainter *P, real x1, real y1,
			       size_t i, size_t j, size_t p1, size_t p2)
{
  // plot the interface vector
  real x2 = PL->x(p2);
  real y2 = PL->y(p2);
  real xr1 = .5*(x1 + x2);
  real yr1 = .5*(y1 + y2);
  real xr2, yr2, xr3, yr3, xr4, yr4;
  real vx = POP->Fnx(i,j);
  real vy = POP->Fny(i,j);
  TurnRight(vx, vy);
  if(p1 < p2) {
    P->setPen(darkGreen);
    xr2 = xr1 + .5*POP->Fnx(i,j);
    yr2 = yr1 + .5*POP->Fny(i,j);
    xr3 = xr1 + .4*POP->Fnx(i,j) + .1*vx;
    yr3 = yr1 + .4*POP->Fny(i,j) + .1*vy;
    xr4 = xr1 + .4*POP->Fnx(i,j) - .1*vx;
    yr4 = yr1 + .4*POP->Fny(i,j) - .1*vy;
  } else {
    P->setPen(red);
    xr2 = xr1 - .5*POP->Fnx(i,j);
    yr2 = yr1 - .5*POP->Fny(i,j);
    xr3 = xr1 - .4*POP->Fnx(i,j) + .1*vx;
    yr3 = yr1 - .4*POP->Fny(i,j) + .1*vy;
    xr4 = xr1 - .4*POP->Fnx(i,j) - .1*vx;
    yr4 = yr1 - .4*POP->Fny(i,j) - .1*vy;
  };
  int xi1, yi1, xi2, yi2, xi3, yi3, xi4, yi4;
  pw->WorldToScreen(xr1, yr1, xi1, yi1);
  pw->WorldToScreen(xr2, yr2, xi2, yi2);
  pw->WorldToScreen(xr3, yr3, xi3, yi3);
  pw->WorldToScreen(xr4, yr4, xi4, yi4);
  P->drawLine(xi1, yi1, xi2, yi2);
  P->drawLine(xi2, yi2, xi3, yi3);
  P->drawLine(xi2, yi2, xi4, yi4);
};

void MQPopPlotter::PlotSegment(MQPlotWidget *pw, QPainter *P, 
			       size_t i, size_t p1, size_t p2)
{
  real x1 = EL->x_middle(i);
  real y1 = EL->y_middle(i);
  real x2 = .5*(PL->x(p1) + PL->x(p2));
  real y2 = .5*(PL->y(p1) + PL->y(p2));
  int xi1, xi2, yi1, yi2;
  pw->WorldToScreen(x1, y1, xi1, yi1);
  pw->WorldToScreen(x2, y2, xi2, yi2);
  P->drawLine(xi1, yi1, xi2, yi2);
};

void MQPopPlotter::PlotBVectors(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  FORALL(i, PL->) {
    size_t p1 = i;
    for (size_t j = 0; j < POP->NumBLinks(i); j++) {
      size_t p2 = POP->BLinkPoint(i, j);
      real x1 = .75*PL->x(p1)+.25*PL->x(p2);
      real y1 = .75*PL->y(p1)+.25*PL->y(p2);
      if (pw->IsIn(x1, y1)) {
	real vx = POP->Bnx(i, j);
	real vy = POP->Bny(i, j);
	TurnLeft(vx, vy);
	real x2 = x1 + .5*POP->Bnx(i, j);
	real y2 = y1 + .5*POP->Bny(i, j);
	real x3 = x1 + .4*POP->Bnx(i,j) + .1*vx;
	real y3 = y1 + .4*POP->Bny(i,j) + .1*vy;
	real x4 = x1 + .4*POP->Bnx(i,j) - .1*vx;
	real y4 = y1 + .4*POP->Bny(i,j) - .1*vy;
	int xi1, yi1, xi2, yi2, xi3, yi3, xi4, yi4;
	pw->WorldToScreen(x1, y1, xi1, yi1);
	pw->WorldToScreen(x2, y2, xi2, yi2);
	pw->WorldToScreen(x3, y3, xi3, yi3);
	pw->WorldToScreen(x4, y4, xi4, yi4);
	P->setPen(darkGreen);
	P->drawLine(xi1, yi1, xi2, yi2);
	P->drawLine(xi2, yi2, xi3, yi3);
	P->drawLine(xi2, yi2, xi4, yi4);
      };
    };
  };
};

void MQPopPlotter::Plot(MQPlotWidget *pw, QPainter *P, const QRect *R)
{
  FORALL(i, PL->) {
    size_t p1 = i;
    real x1 = PL->x(p1);
    real y1 = PL->y(p1);
    for (size_t j = 0; j < POP->NumFLinks(i); j++) {
      size_t p2 = POP->FLinkPoint(i, j);
      if (pw->IsIn(.5*(PL->x(p1) + PL->x(p2)), .5*(PL->y(p1) + PL->y(p2)))) {
	PlotFVector(pw, P, x1, y1, i, j, p1, p2);
      };
    };
  };
  FORALL(i, EL->) {
    if (ElementInside(pw, i, R)) {
      for (size_t j = 0; j < EL->NumNodes(i); j++) {
	size_t p1 = EL->PermNode(i, j);
	size_t p2 = EL->PermNode(i, j+1);
	P->setPen(QPen(blue, 1, DotLine));
	PlotSegment(pw, P, i, p1, p2);
      };
    };
  };
  PlotBVectors(pw, P, R);
};

#endif













