// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!


#define POPLIST PopList2D

#include <mouse_config.h>

#include "MouseMain.hh"
#include <TList.hh>

#include <MObjectBuilder.hh>
#include <FParser.hh>
#include <CreateObject.hh>
#include <ElementList.hh>
#include <GroupAction.hh>
#include <IntervalAction.hh>
#include <MIntList.hh>
#include <MRealList.hh>
#include <MStringList.hh>
#include <Parameter.hh>
#include <StepAction.hh>
#include <NopAction.hh>
#include <DrawCircle.hh>
#include <DrawPolygon.hh>
#include <Triangulator2D.hh>
#include <Exec.hh>
#include <Mesh2D.hh>
#include <TriRefinement2D.hh>
#include <Delaunay2D.hh>
#include <RelaxNetDensity2D.hh>
#include <MeshSmoother2D.hh>
#include <MeshSaver2D.hh>
#include <WriteMeshInfo2D.hh>
#include <ReadMeshInfo2D.hh>
#include <TFEdgeCalculator2D.hh>
#include <TPointCalculator.hh>
#include <TBPointCalculator.hh>
#include <MusclProjector2D1.hh>
#include <MusclProjector2D2.hh>
#include <MusclProjectorLimVA2D.hh>
#include <MusclProjectorLimMM32D.hh>
#include <PopList2D.hh>
#include <BoundaryPointList.hh>
#include <POperation.hh>
#include <MclPOperation.hh>
#include <MclBPOperation.hh>
#include <ForwardEuler.hh>
#include <CflManager.hh>
#include <GroupPointCalculator.hh>
#include <PrintMaximum.hh>
#include <PrintMaximalDiff.hh>
#include <WriteMaximalDiff.hh>
#include <CalcGradGreen2D.hh>
#include <MeshStep2D.hh>
#include <CompressibleBFlux2D.hh>
#include <CompressibleWFlux2D.hh>
#include <CompressibleTStep2D.hh>
#include <CompressibleInit2D.hh>
#include <Ausm2D.hh>
#include <CompressibleTWall2D.hh>
#include <CompressibleSWall2D.hh>
#include <CompressibleInOutCh2D.hh>
#include <CompressibleViscFlux2D.hh>
#include <RungeKutta.hh>
#include <WriteTecplotFile2D.hh>
#include <RoeTurkel2D.hh>
#include <Roe2D.hh>
#include <DataReader.hh>
#include <DataSaver.hh>
#include <ReadPicFile.hh>
#include <LoadPolygon2D.hh>
#include <EquiCartBlock2D.hh>
#include <CircularBlock2D.hh>
#include <MultiGridAgent.hh>
#include <MultiGridRestriction.hh>
//#include <MultiGridCoarseGridSetup.hh>
//#include <MultiGridTransferToCoarse.hh>
//#include <MultiGridTransferToFine.hh>
#include <GenericSourceTerm.hh>
#include <SetFieldToZero.hh>

#include <CoarseGridToBoundary.hh>
#include <CoarseGridFromBoundary.hh>
#include <CoarseGridWeightedFromBoundary.hh>
#include <CoarseGridReduceBoundary.hh>

#include <CopyField.hh>
#include <SubstractField.hh>
#include <SetFieldToConst.hh>
#include <AddField.hh>

#include <MultiGridRestriction.hh>
#include <MultiGridWeightedRestriction.hh>
#include <MultiGridProlongation.hh>

#include <CompressibleInOutFlux2D.hh>

#include <WriteMeshForSINUS.hh>

#ifdef WITH_QT
#include <MQApplication.hh>
#include <CreatePlotWindow.hh>
#include <MQMenuAction.hh>
#include <Repaint.hh>
#include <MQMeshPlotter.hh>
#include <MQPointInfoPlotter.hh>
#include <MQMultiGridPointInfoPlotter.hh>
#include <MQPointVarInfoPlotter.hh>
#include <MQPopPlotter.hh>
#include <MQMultiGridPopPlotter.hh>
#include <VolIsoValue.hh>
#include <ChDistIsoValue.hh>
#include <MQIsolinePlotter.hh>
#include <MQBoundaryPlotter.hh>
#include <MQBoundaryNVecPlotter.hh>
#include <MQTreeControlWindow.hh>
#include <CreateTreeControlWindow.hh>
#include <IsoVarValue.hh>
#include <IsoVarDiffValue.hh>
#include <CompressibleVelocityVector2D.hh>
#include <MQVectorPlotter.hh>
#include <CompressiblePressureValue2D.hh>
#include <MultiGridIsoVarValue.hh>
#include <MQCellInfoPlotter.hh>
#include <MQMultiGridCellInfoPlotter.hh>
#endif

#include <Extensions1.hh>

Communicator TopObjectOfMouse;
size_t default_num_threads;

MouseMain::MouseMain() : MObject(carg_t(MouseMain::Top()))
{
  AddClassName();
  StartStdMcl();
  default_num_threads = 1;
  file = new TList<string> (5, 5, "");
  parsing_file = true;
};  

void MouseMain::StartStdMcl()
{
  // initializing builder objects for the standard MCL commands
  Extensions1::Register();

  new MObjectBuilder<CreateObject>
    (this,"CreateObject");
  new MObjectBuilder<GroupAction>
    (this,"GroupAction");
  new MObjectBuilder<ElementList>
    (this,"ElementList");
  new MObjectBuilder<IntervalAction>
    (this,"IntervalAction");
  new MObjectBuilder<Parameter>
    (this,"Parameter");
  new MObjectBuilder<PointList>
    (this,"PointList");
  new MObjectBuilder<StepAction>
    (this,"StepAction");
  new MObjectBuilder<VarList>
    (this,"VarList");
  new MObjectBuilder<MIntList>
    (this,"MIntList");
  new MObjectBuilder<MRealList>
    (this,"MRealList");
  new MObjectBuilder<MStringList>
    (this,"MStringList");
  new MObjectBuilder<NopAction>
    (this,"NopAction");
  new MObjectBuilder<DrawCircle>
    (this,"DrawCircle");
  new MObjectBuilder<DrawPolygon>
    (this,"DrawPolygon");
  new MObjectBuilder<Triangulator2D>
    (this,"Triangulator2D");
  new MObjectBuilder<Exec>
    (this,"Exec"); 
  new MObjectBuilder<Mesh2D>
    (this,"Mesh2D");
  new MObjectBuilder<TriRefinement2D>
    (this,"TriRefinement2D");
  new MObjectBuilder<Delaunay2D>
    (this,"Delaunay2D");
  new MObjectBuilder<RelaxNetDensity2D>
    (this,"RelaxNetDensity2D");
  new MObjectBuilder<MeshSmoother2D>
    (this,"MeshSmoother2D");
  new MObjectBuilder<MeshSaver2D>
    (this,"MeshSaver2D");
  new MObjectBuilder<WriteMeshInfo2D>
    (this,"WriteMeshInfo2D");
  new MObjectBuilder<ReadMeshInfo2D>
    (this,"ReadMeshInfo2D");
  new MObjectBuilder<TFEdgeCalculator2D<MusclProjector2D1,POPLIST> >
    (this, "MusclProjector2D1");
  new MObjectBuilder<TFEdgeCalculator2D<MusclProjector2D2,POPLIST> >
    (this, "MusclProjector2D2");
  new MObjectBuilder<TFEdgeCalculator2D<MusclProjectorLimVA2D,POPLIST> >
    (this, "MusclProjectorLimVA2D");
  new MObjectBuilder<TFEdgeCalculator2D<MusclProjectorLimMM32D,POPLIST> >
    (this, "MusclProjectorLimMM2D");
  new MObjectBuilder<POPLIST>
    (this, "PopList2D");
  new MObjectBuilder<BoundaryPointList>
    (this, "BoundaryPointList");

  new MObjectBuilder<MclPOperation>
    (this, "MclPOperation");
  new MObjectBuilder<MclBPOperation>
    (this, "MclBPOperation");
  new MObjectBuilder<MclPOperation>
    (this, "forall_p");
  new MObjectBuilder<MclBPOperation>
    (this, "forall_bp");

  new MObjectBuilder<ForwardEuler<POPLIST> >
    (this, "ForwardEuler");
  new MObjectBuilder<CflManager>
    (this, "CflManager");
  new MObjectBuilder<GroupPointCalculator>
    (this, "GroupPointCalculator");
  new MObjectBuilder<PrintMaximum>
    (this,"PrintMaximum");
  new MObjectBuilder<PrintMaximalDiff>
    (this,"PrintMaximalDiff");
  new MObjectBuilder<WriteMaximalDiff>
    (this,"WriteMaximalDiff");
  new MObjectBuilder<CalcGradGreen2D<POPLIST> >
    (this,"CalcGradGreen2D");
  new MObjectBuilder<MeshStep2D>
    (this,"MeshStep2D");
  new MObjectBuilder<TBEdgeCalculator2D<CompressibleBFlux2D,POPLIST> >
    (this, "CompressibleBFlux2D");
  new MObjectBuilder<TBEdgeCalculator2D<CompressibleInOutFlux2D,POPLIST> >
    (this, "CompressibleInOutFlux2D");
  new MObjectBuilder<TBEdgeCalculator2D<CompressibleWFlux2D,POPLIST> >
    (this, "CompressibleWFlux2D");
  new MObjectBuilder<TFEdgeCalculator2D<Ausm2D,POPLIST> >
    (this, "Ausm2D");
  new MObjectBuilder<TFEdgeCalculator2D<RoeTurkel2D,POPLIST> >
    (this, "RoeTurkel2D");
  new MObjectBuilder<TFEdgeCalculator2D<Roe2D,POPLIST> >
    (this, "Roe2D");
  new MObjectBuilder<TPointCalculator<CompressibleTStep2D> >
    (this, "CompressibleTStep2D");
  new MObjectBuilder<TPointCalculator<CompressibleInit2D> >
    (this, "CompressibleInit2D");
  new MObjectBuilder<TBPointCalculator<CompressibleInit2D> >
    (this, "CompressibleBFix2D");
  new MObjectBuilder<TBPointCalculator<CompressibleTWall2D> >
    (this, "CompressibleTWall2D");
  new MObjectBuilder<TBPointCalculator<CompressibleSWall2D> >
    (this, "CompressibleSWall2D");
  new MObjectBuilder<TBPointCalculator<CompressibleInOutCh2D> >
    (this, "CompressibleInOutCh2D");
  new MObjectBuilder<RungeKutta<POPLIST> >
    (this, "RungeKutta");
  new MObjectBuilder<TFEdgeCalculator2D<CompressibleViscFlux2D,POPLIST> >
    (this, "CompressibleViscFlux2D");
  new MObjectBuilder<WriteTecplotFile2D>
    (this, "WriteTecplotFile2D");
  new MObjectBuilder<DataReader>
    (this, "DataReader");
  new MObjectBuilder<DataSaver>
    (this, "DataSaver");
  new MObjectBuilder<ReadPicFile>
    (this, "ReadPicFile");
  new MObjectBuilder<LoadPolygon2D>
    (this, "LoadPolygon2D");
  new MObjectBuilder<EquiCartBlock2D<POPLIST> >
    (this, "EquiCartBlock2D");
  new MObjectBuilder<CircularBlock2D<POPLIST> >
    (this, "CircularBlock2D");
  new MObjectBuilder<MultiGridAgent<POPLIST> >
    (this, "MultiGridAgent");
  new MObjectBuilder<TPointCalculator<MultiGridProlongation<POPLIST> > >
    (this, "MultiGridProlongation");
  new MObjectBuilder<TPointCalculator<MultiGridRestriction<POPLIST> > >
    (this, "MultiGridRestriction");
  new MObjectBuilder<TPointCalculator<MultiGridWeightedRestriction<POPLIST> > >
    (this, "MultiGridWeightedRestriction");

//   new MObjectBuilder<MultiGridCoarseGridSetup<POPLIST> >
//     (this, "MultiGridCoarseGridSetup");
//   new MObjectBuilder<MultiGridTransferToCoarse<POPLIST> >
//     (this, "MultiGridTransferToCoarse");
//   new MObjectBuilder<MultiGridTransferToFine<POPLIST> >
//     (this, "MultiGridTransferToFine");

  new MObjectBuilder<TPointCalculator<GenericSourceTerm> >
    (this, "GenericSourceTerm");
  new MObjectBuilder<SetFieldToZero<POPLIST> >
    (this, "SetFieldToZero");

  new MObjectBuilder<TBPointCalculator<CoarseGridToBoundary<POPLIST> > >
    (this, "CoarseGridToBoundary");
  new MObjectBuilder<TBPointCalculator<CoarseGridWeightedFromBoundary<POPLIST> > >
    (this, "CoarseGridWeightedFromBoundary");
  new MObjectBuilder<TBPointCalculator<CoarseGridFromBoundary<POPLIST> > >
    (this, "CoarseGridFromBoundary");
  new MObjectBuilder<TPointCalculator<CoarseGridReduceBoundary<POPLIST> > >
    (this, "CoarseGridReduceBoundary");

  new MObjectBuilder<TPointCalculator<CopyField> >
    (this, "CopyField");
  new MObjectBuilder<TBPointCalculator<CopyField> >
    (this, "CopyBField");
  new MObjectBuilder<TPointCalculator<AddField> >
    (this, "AddField");
  new MObjectBuilder<TBPointCalculator<AddField> >
    (this, "AddBField");
  new MObjectBuilder<TPointCalculator<SubstractField> >
    (this, "SubstractField");
  new MObjectBuilder<TBPointCalculator<SubstractField> >
    (this, "SubstractBField");
  new MObjectBuilder<TPointCalculator<SetFieldToConst> >
    (this, "SetFieldToConst");
  new MObjectBuilder<TBPointCalculator<SetFieldToConst> >
    (this, "SetBFieldToConst");

  new MObjectBuilder<WriteMeshForSINUS>
    (this, "WriteMeshForSINUS");


#ifdef WITH_QT
  new MObjectBuilder<MQApplication>
    (this,"MQApplication");
  new MObjectBuilder<CreatePlotWindow>
    (this,"CreatePlotWindow");
  new MObjectBuilder<MQMenuAction>
    (this,"MQMenuAction");
  new MObjectBuilder<Repaint>
    (this,"Repaint");
  new MObjectBuilder<MQMeshPlotter>
    (this,"MQMeshPlotter");
  new MObjectBuilder<MQPointInfoPlotter>
    (this,"MQPointInfoPlotter");
  new MObjectBuilder<MQMultiGridPointInfoPlotter>
    (this,"MQMultiGridPointInfoPlotter");
  new MObjectBuilder<MQPointVarInfoPlotter>
    (this,"MQPointVarInfoPlotter");
  new MObjectBuilder<MQPopPlotter>
    (this,"MQPopPlotter");
  new MObjectBuilder<MQMultiGridPopPlotter>
    (this,"MQMultiGridPopPlotter");
  new MObjectBuilder<VolIsoValue>
    (this,"VolIsoValue");
  new MObjectBuilder<ChDistIsoValue>
    (this,"ChDistIsoValue");
  new MObjectBuilder<MQIsolinePlotter>
    (this,"MQIsolinePlotter");
  new MObjectBuilder<MQBoundaryPlotter>
    (this,"MQBoundaryPlotter");
  new MObjectBuilder<MQBoundaryNVecPlotter>
    (this,"MQBoundaryNVecPlotter");
  new MObjectBuilder<MQTreeControlWindow>
    (this,"MQTreeControlWindow");
  new MObjectBuilder<CreateTreeControlWindow>
    (this,"CreateTreeControlWindow");
  new MObjectBuilder<IsoVarValue>
    (this,"IsoVarValue");
  new MObjectBuilder<MultiGridIsoVarValue<POPLIST> >
    (this,"MultiGridIsoVarValue");
  new MObjectBuilder<IsoVarDiffValue>
    (this,"IsoVarDiffValue");
  new MObjectBuilder<CompressibleVelocityVector2D>
    (this, "CompressibleVelocityVector2D");
  new MObjectBuilder<MQVectorPlotter>
    (this, "MQVectorPlotter");
  new MObjectBuilder<CompressiblePressureValue2D>
    (this, "CompressiblePressureValue2D");
  new MObjectBuilder<MQCellInfoPlotter>
    (this, "MQCellInfoPlotter");
  new MObjectBuilder<MQMultiGridCellInfoPlotter>
    (this, "MQMultiGridCellInfoPlotter");
#endif
};

void MouseMain::main(Parser *P)
{
  TList<MObject*> obj(5, 5, NULL);
  while (!P->eof()) {
    Top()->BuildObject(obj.NewEntry(), carg_t(Top(), P));
  };
  Top()->CollectGarbage();
  parsing_file = false;
  cout << "\n\nno errors found in the control file\n" << endl;
  FORALL(i, obj.) {
    Action *act = dynamic_cast<Action*>(obj.At(i));
    if (act == NULL) {
      cout << "I cannot execute an object of this type:" << endl;
      cout << obj.At(i)->TopClassName() << endl;
      exit(EXIT_FAILURE);
    };
    act->Act();
  };
};


void MouseMain::StartMouse(int argc, char* argv[])
{
  if (argc != 3) {
    cout << "usage: " << argv[0] << " port filename" << endl;
  } else {
    Top()->argc = argc;
    Top()->argv = argv;
    istrstream s(argv[1]);
    unsigned short port;
    s >> port;
    Top()->SetupServer(port);
    FParser P(Top(), argv[argc - 1]);
    if (P.fail()) {
      cout << "error opening '" << argv[argc - 1] << "'" << endl;
    } else {
      main(&P);
    };
  };
};

















