// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "POperation.hh"

POperation::POperation(ConstructArg arg) : Action(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  initialized = false;
};

void POperation::Init(size_t a_num_threads, size_t a_group_size)
{
  group_size = a_group_size;
  if (group_size == 0) group_size = 500;
  num_threads = a_num_threads;
  if (num_threads == 0) num_threads = MouseMain::NumThreads();
  point_tmp = new TList<VarContainer*>(5, 5, NULL);
  edge_tmp = new TList<VarContainer*>(point_tmp, NULL);
  calc = new TList<PointCalculator*>(point_tmp, NULL);
  for (size_t i = 0; i < num_threads; i++) {
    size_t i_new = point_tmp->AddEntry();
    current_thread = i_new;
    point_tmp->At(i_new) = new VarContainer(this);
    edge_tmp->At(i_new) = new VarContainer(this);
    calc->At(i_new) = NULL;
  };
  initialized = true;
};

POperation::~POperation()
{
  for (size_t i = 0; i < num_threads; i++) {
    current_thread = i;
    delete point_tmp->At(i);
    delete edge_tmp->At(i);
    delete calc->At(i);
  };
  delete point_tmp;
  delete edge_tmp;
  delete calc;
};

void POperation::Act()
{
  if (!initialized) Error("'POperation' has not been initialized!");
  size_t i_begin = PL->begin();
  size_t i_end;
  while (i_begin < PL->end()) {
    i_end = i_begin + group_size;
    if (i_end > PL->end()) i_end = PL->end();
    calc->At(0)->CalcGroup(i_begin, i_end);
    i_begin = i_end;
  };
};







