// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "Parameter.hh"

//
//.. constructor
//
Parameter::Parameter (ConstructArg arg) : MObject (arg)
{
  AddClassName();
  real_param = new TKeyList<string, real> (10, 10, "", 0);
  flags = new TKeyList<string, bool> (10, 10, "", false);
  int_param = new TKeyList<string, int> (10, 10, "", 0);
  var_oo = new TList<real>(10, 10);
  string name, word;
  while (!arg.P->eof()) {
    word = arg.P->ReadString();

    if (word == "int") {
      // look for integer parameter definitions
      long int value;
      name = arg.P->ReadString();
      value = arg.P->ReadInt();
      //if (int_param->Find(name) != -1) 
      //Error("integer parameter '" + name + "' defined twice");
      bool found = true;
      try { 
	int_param->Find(name);
      }
      catch (NotFound_error) {
	found = false;
      };
      if (found) Error("integer parameter '" + name + "' defined twice");
      int_param->NewEntry(name) = value;

    } else if (word == "real") {
      // look for real parameter definitions
      real value;
      name = arg.P->ReadString();
      value = arg.P->ReadReal();
      //if (real_param->Find(name) != -1) 
      //Error("real parameter '" + name + "' defined twice");
      bool found = true;
      try {
	real_param->Find(name);
      }
      catch (NotFound_error) {
	found = false;
      };
      if (found) Error("real parameter '" + name + "' defined twice");
      real_param->NewEntry(name) = value;

    } else if (word == "flag") {
      // look for flag definitions
      bool value = false;
      name = arg.P->ReadString();
      word = arg.P->ReadString();
      if ((word == "true") || (word == "yes")) {
	value = true;
      } else if ((word == "false") || (word == "no")) {
	value = false;
      } else {
	Error ("'true/yes' or 'false/no' expected");
      };
      //if (flags->Find(name) != -1) 
      //Error("flag '" + name + "' defined twice");
      bool found = true;
      try {
	flags->Find(name);
      }
      catch (NotFound_error) {
	found = false;
      };
      if (found) Error("flag '" + name + "' defined twice");
      flags->NewEntry(name) = value;

    } else {
      Error ("'" + word + "': unknown command");
    };
  };
};

//
//.. RealParam
//
real Parameter::RealParam (string name)
{
  //if (real_param->Find(name) == -1) Error(name + " undefined");
  real param = 0;;
  try {
    param = real_param->At(name);
  }
  catch (NotFound_error) {
    Error(name + " undefined");
  };
  return param;
};

//
//.. Flag
//
bool Parameter::Flag (string name)
{
  //if (flags->Find(name) == -1) Error(name + " undefined");
  //return flags->At(name);
  bool param = false;
  try {
    param = flags->At(name);
  }
  catch (NotFound_error) {
    Error(name + " undefined");
  };
  return param;
};

//
//.. Flag
//
int Parameter::IntParam (string name)
{
  //if (int_param->Find(name) == -1) Error(name + " undefined");
  //return int_param->At(name);
  long int param = 0;
  try {
    param = int_param->At(name);
  }
  catch (NotFound_error) {
    Error(name + " undefined");
  };
  return param;
};

//
//.. SendIntParam
//
void Parameter::SendIntParam(Event &E, string name)
{
  E.AddOutInt(IntParam(name));
};

//
//.. SendRealParam
//
void Parameter::SendRealParam(Event &E, string name)
{
  E.AddOutReal(RealParam(name));
};

//
//.. HandleEvent
//
void Parameter::HandleEvent(Event &E)
{
  MObject::HandleEvent(E);
  if (!E.Done()) {
    if (E.name == "Need.ValueOnMesh") {
      if (E.InString(0) == MeshID()) {
	E.React(this);
	//if (int_param->Find(E.InString(1)) != -1) SendIntParam(E, E.InString(1));
	//if (real_param->Find(E.InString(1)) != -1) SendRealParam(E, E.InString(1));
	bool found;
	// check for int parameter
	found = true;
	try {
	  int_param->Find(E.InString(1));
	}
	catch (NotFound_error) {
	  found = false;
	};
	if (found) SendIntParam(E, E.InString(1));
	// check for real parameter
	found = true;
	try {
	  real_param->Find(E.InString(1));
	}
	catch (NotFound_error) {
	  found = false;
	};
	if (found) SendRealParam(E, E.InString(1));
      };
    };
  };
};

//
//.. InitVars
//
void Parameter::InitVars(VarList *VL)
{
  VarList::var2_t var = VL->Get2DVar("new");
  FORALL(i, VL->) {
    for (size_t j = 0; j < VL->NumVars(); j++) var[j][i] = 0.0;
  };
};









