// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(Parser_HH)
#define Parser_HH

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>

class Parser;

#include <TKeyList.hh>
#include "MObject.hh"
#include "ConstructArg.hh"

/** */
class Parser : public MObject
{
protected:
  /** */ TKeyList<string, MObject*> *VAR;

public:
  /** */ Parser (MObject *a_parent);

  /** */ virtual void Reset () = 0;

  /** read an integer value. */
  virtual int ReadInt () = 0;

  /** read an unsigned integer value. */
  virtual size_t ReadIndex () = 0;

  /** read a string. */
  virtual string ReadString () = 0;

  /** read a floating point value. */
  virtual real ReadReal () = 0;

  /** read a boolean value from the words true or false. */
  virtual bool ReadBool();

  /** create a subparser with the help of enclosing brackets. */
  virtual Parser* SubParser () = 0;

  /** @return the eof flag of the input stream of the Parser. */
  virtual bool eof() = 0;

  /** */ virtual MObject* ResolveVar();

  /** get the MObject pointer of a variable definition. */
  MObject* Var(string var_name);

  /** add a variable definition. */
  void AddVar(string var_name, MObject *var_obj);
};



#endif











