// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(POINTCALCULATOR_HH)
#define POINTCALCULATOR_HH

class PointCalculator;

#include <MObject.hh>
#include <POperation.hh>
#include <PointList.hh>

/**
   This is the basis for operations on a subset of points. 
   A point-iterator object (e.g. StdPointAction) is responsible to call the
   calculator several times, so that all points will be computed. 
   If you are using a thread-based parallel point-iterator object
   one calculator for every thread will be created. 
   The subsets have to be continuous in the PointList. 
   Right now it is not even allowed to have deleted entries in the PointList. 
   It is however planned to allow this, especially since the PointList 
   is derived from List, which was designed to handle exactly this. 
*/
class PointCalculator : public MObject
{
protected:
  /** the list of all points */
  PointList *PL;

  /** the object iterating over all points of a mesh */
  POperation *p_operation;

  /** the number of the thread this calculator is running in */
  size_t thread_id;

  /** point based temporary data */
  VarContainer *point_tmp;

  /** edge based temporary data */
  VarContainer *edge_tmp;
  
public:
  /** */ PointCalculator(ConstructArg arg);

  /** CalcGroup performs the computation of the point-group.
      In most cases however this method should not be overwritten
      directly. Instead a generic calculator class such as TPointCalculator
      can be used.
      @param i_begin the first point of the group
      @param i_end the first point in the PointList not belonging
      to this subset anymore. */
  virtual void CalcGroup(size_t i_begin, size_t i_end) = 0;

  /** Get the thread id number of this calculator.
      @return the thread id number */
  size_t ThreadID() { return thread_id; };
};

#endif















