// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(POPLIST2D_HH)
#define POPLIST2D_HH

class PopList2D;

#include <MObject.hh>
#include <TList.hh>
#include <PointList.hh>
#include <BoundaryPointList.hh>
#include <ElementList.hh>
#include <Operator.hh>

/** */
class PopList2D : public MObject
{
protected:
  struct link_t {
    unsigned short int count;
    size_t point;
    real nx, ny;
    link_t() { count = 0; point = 0; nx = 0; ny = 0; };
    link_t(size_t a_point) { point = a_point; };
    bool operator==(const link_t &link) { return point == link.point; };
  };

  struct reset_op : public Operator {
    reset_op(carg_t a) : Operator(a) { AddClassName(); };
    void operate(size_t i, size_t p);
  };

  struct calcvol_op : public Operator {
    real ax, ay, zylvol, qarea, i_pro_dist;
    calcvol_op(carg_t a) : Operator(a) { AddClassName(); };
    void operate(size_t ie, size_t ip, size_t from, size_t to, 
		 real nx, real ny, bool mod_from, bool mod_to);
  };

  struct postop_op : public Operator {
    postop_op(carg_t a) : Operator(a) { AddClassName(); };
    void operate(size_t i, size_t p);
  };

  struct normvec_op : public Operator {
    BoundaryPointList *BPL;
    PopList2D *POP;
    normvec_op(carg_t a);
    void operate(size_t i, size_t p);
  };

  friend class normvec_op;
    

  /** */ PointList *PL;
  /** */ ElementList *EL;
  /** */ BoundaryPointList *BPL;
  /** */ TList<TList<link_t>*> *flink;
  /** */ TList<TList<link_t>*> *blink;
  /** */ TList<size_t> *group_id;

  /** */ void CalcNVec(size_t i);

public:
  /** */ PopList2D(ConstructArg arg);
  /** */ void DelAllSublists();
  /** */ virtual void Update();
  /** */ size_t NumFLinks(size_t i) {
    if (flink->At(i) == NULL) return 0;
    return flink->At(i)->NumEntries(); 
  };
  /** */ size_t NumBLinks(size_t i) { 
    if (PL->code(i) == 0) return 0;
    if (blink->At(BPL->BIndex(i)) == NULL) return 0;
    return blink->At(BPL->BIndex(i))->NumEntries(); 
  }
  /** */ size_t FLinkPoint(size_t i, size_t j) { 
    return flink->At(i)->At(j).point; 
  };
  /** */ size_t BLinkPoint(size_t i, size_t j) {
    return blink->At(BPL->BIndex(i))->At(j).point;
  };
  /** */ real Fnx(size_t i, size_t j) { 
    return flink->At(i)->At(j).nx; 
  }; 
  /** */ real Fny(size_t i, size_t j) { 
    return flink->At(i)->At(j).ny; 
  }; 
  /** */ real Bnx(size_t i, size_t j) { 
    return blink->At(BPL->BIndex(i))->At(j).nx;
  }; 
  /** */ real Bny(size_t i, size_t j) { 
    return blink->At(BPL->BIndex(i))->At(j).ny;
  }; 
  /** */ size_t GroupID(size_t i) { return group_id->At(i); };


  // additional methods since version 1.0.1
  // they became necessary during the multi-grid development

  /** */ void AddFLink(size_t p1, size_t p2, real nx, real ny);
  /** */ void AddBLink(size_t p1, size_t p2, real nx, real ny);
  /** */ void AddBLink(size_t p1, size_t p2);

  /** */ size_t FindFLink(size_t p1, size_t p2);
  /** */ size_t FindBLink(size_t p1, size_t p2);
  /** */ void SetFnx(size_t i, size_t j, real nx);
  /** */ void SetFny(size_t i, size_t j, real ny);
  /** */ void SetBnx(size_t i, size_t j, real nx);
  /** */ void SetBny(size_t i, size_t j, real ny);

  /** */ void Print(ostream &s);

  /** */ void UpdateMetric();
};

#endif













