// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 19 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "PrintMaxRes.hh"

PrintMaxRes::PrintMaxRes(carg_t arg)
  : Action(arg)
{
  AddClassName();
  VL = FindObject<VarList>(false, true);
  MRES = ResolveVar<ComputeMaxRes>(arg);
  scale_max = new real[VL->NumVars()];
  scale_l2 = new real[VL->NumVars()];
  first_call = true;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    scale_max[iv] = 1e-30;
    scale_l2[iv] = 1e-30;
  };
};

PrintMaxRes::~PrintMaxRes()
{
  delete [] scale_max;
  delete [] scale_l2;
}

void PrintMaxRes::Reset()
{
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    scale_max[iv] = 1e-30;
    scale_l2[iv] = 1e-30;
  };
}

void PrintMaxRes::Act()
{
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    if (MRES->MaxRes(iv) > scale_max[iv]) scale_max[iv] = MRES->MaxRes(iv);
    if (MRES->L2Res(iv) > scale_l2[iv]) scale_l2[iv] = MRES->L2Res(iv);
  };
  cout << "\n======================================" << endl;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    string vname = "norm_max(res(" + VL->VarName(iv) + "))";
    while (vname.size() < 20) vname += ' ';
    cout << vname << " = " << MRES->MaxRes(iv)/scale_max[iv] << endl;
  };
  cout << "--------------------------------------" << endl;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    string vname = "max(res(" + VL->VarName(iv) + "))";
    while (vname.size() < 20) vname += ' ';
    cout << vname << " = " << MRES->MaxRes(iv) << endl;
  };
  cout << "--------------------------------------" << endl;
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    string vname = "L2_norm(res(" + VL->VarName(iv) + "))";
    while (vname.size() < 20) vname += ' ';
    cout << vname << " = " << MRES->L2Res(iv)/scale_l2[iv] << endl;
  };
  cout << "======================================\n" << endl;
};

