// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "RelaxNetDensity2D.hh"
#include <geometry2D.hh>

//
//.. constructor
//
RelaxNetDensity2D::RelaxNetDensity2D(ConstructArg arg) : Mesh2DAction(arg)
{
  AddClassName();
  step_limit = arg.P->ReadIndex();
  EL = FindObject<ElementList>(false, true);
  PEL = FindObject<PelList2D>(false, true);
  PL = FindObject<PointList>(false, true);
};

//
//.. Act
//
void RelaxNetDensity2D::Act()
{
  //int i, j;
  //int i_list, i_point;
  // iterative loop for field nodes
  real step;
  real xp, yp, xa, ya, xb, yb;
  real dAx, dAy, us, ls;
  real nd, nd_pa, nd_pb;
  //int n;
  size_t pa, pb;
  int counter = 0;
  real DSQ, CROSS;
  bool do_step;
  size_t i, n;
  do {
    FORALL (i_point, PL->) {
      if ((PL->code(i_point) == 0) 
	  && !MESH->IsFixed(i_point)
	  && MESH->IsMarked(i_point)) {
	do_step = true;
	xp = PL->x(i_point);
	yp = PL->y(i_point);
	nd = MESH->ND(i_point);
	// initialize upper and lower sums
	us = 0;
	ls = 0;
	for (i = 0; i < PEL->Size(i_point); i++) {
	  n = MESH->FindNode(PEL->At(i_point, i), i_point);
	  pa = EL->PermNode(PEL->At(i_point, i), n + 1);
	  pb = EL->PermNode(PEL->At(i_point, i), n - 1);
	  xa = PL->x(pa);
	  ya = PL->y(pa);
	  xb = PL->x(pb);
	  yb = PL->y(pb);
	  nd_pa = MESH->ND(pa);
	  nd_pb = MESH->ND(pb);
	  // control boundary segment length
	  dAx = yb - ya;
	  dAy = xa - xb;
	  // stretching for vector P->.5*(Pa+Pb)
	  // iteration formulation according to Dr Vilsmeier
	  DSQ = SQR(dAx) + SQR(dAy);
	  CROSS = (xa - xp)*(yb - yp) - (xb - xp)*(ya - yp);
	  //cout << "CROSS/DSQ = " << CROSS/DSQ << endl;
	  if (CROSS/DSQ > 1e-4) {
	    // if area of partial triangle is not zero,
	    // compute the contributions of this partial triangle
	    us += .5*(nd_pa + nd_pb) * DSQ/CROSS;
	    ls += DSQ/CROSS;
	  } else {
	    // do not compute this point in this step,
	    // if partial triangle has a zero area !
	    do_step = false;
	    break;
	  };
	};
	if (do_step) {
	  step = (us/ls) - nd;
	  MESH->SetNetDensity(i_point, MESH->ND(i_point) + 1.3 * step);
	};
      };
    };
    counter++;
  } while (counter < step_limit);  
};







