// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef TWFlux_HH
#define TWFlux_HH

class TWFlux;

#include <CompressibleOperator2D.hh>
#include <geometry2D.hh>

/** */
class TWFlux : public CompressibleOperator2D
{
protected:
  /** */ VarContainer::var1_t drho_dx;
  /** */ VarContainer::var1_t drhou_dx;
  /** */ VarContainer::var1_t drhov_dx;
  /** */ VarContainer::var1_t drhoE_dx;
  /** */ VarContainer::var1_t drho_dy;
  /** */ VarContainer::var1_t drhou_dy;
  /** */ VarContainer::var1_t drhov_dy;
  /** */ VarContainer::var1_t drhoE_dy;

public:
  /** */ TWFlux(carg_t arg);
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

inline void TWFlux::operate(size_t ie, size_t ip, 
			    size_t from, size_t to, 
			    real nx, real ny, 
			    bool mod_from, bool mod_to)
{
  real a = DIST(0,0,nx,ny);
  real nx0 = nx/a;
  real ny0 = ny/a;
  if (mod_from) {
    real b = .25*PL->Vol(from)/a;
    real vx = -nx0*b;
    real vy = -ny0*b;
    real rho_h = rho[from] + vx*drho_dx[from] + vy*drho_dy[from];
    real rhou_h = rhou[from] + vx*drhou_dx[from] + vy*drhou_dy[from];
    real rhov_h = rhov[from] + vx*drhov_dx[from] + vy*drhov_dy[from];
    real rhoE_h = rhoE[from] + vx*drhoE_dx[from] + vy*drhoE_dy[from];
    real p = gamma1 * (rhoE_h - 0.5 * (rhou_h * rhou_h + rhov_h * rhov_h) / rho_h);
    res_rhou[from] += p * nx;
    res_rhov[from] += p * ny;
  };
  if (mod_to) {
    real b = .25*PL->Vol(to)/a;
    real vx = -nx0*b;
    real vy = -ny0*b;
    real rho_h = rho[to] + vx*drho_dx[to] + vy*drho_dy[to];
    real rhou_h = rhou[to] + vx*drhou_dx[to] + vy*drhou_dy[to];
    real rhov_h = rhov[to] + vx*drhov_dx[to] + vy*drhov_dy[to];
    real rhoE_h = rhoE[to] + vx*drhoE_dx[to] + vy*drhoE_dy[to];
    real p = gamma1 * (rhoE_h - 0.5 * (rhou_h * rhou_h + rhov_h * rhov_h) / rho_h);
    res_rhou[to] += p * nx;
    res_rhov[to] += p * ny;
  };
};

#endif












