// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "TriRefinement2D.hh"
#include <PointList.hh>
#include <VarList.hh>
#include <geometry2D.hh>

TriRefinement2D::TriRefinement2D(ConstructArg arg) : Mesh2DAction(arg) 
{
  AddClassName();
  l_max = arg.P->ReadReal();
};

void TriRefinement2D::Act()
{
  ElementList *EL = FindObject<ElementList>(false, true);
  PelList2D *PEL = FindObject<PelList2D>(false, true);
  PointList *PL = FindObject<PointList>(false, true);
  VarList *VL = LookForObject<VarList>(false, true);
  VarContainer::var2_t var;
  if (VL != NULL) var = VL->Get2DVar("new");
  size_t i_element;
  size_t i_node, n, ivar;
  size_t p1, p2, e_left, e_right, pA, pB, pC, pD, ne;
  size_t e_new, p_new;
  real length, nd;
  real x1_t, y1_t, x2_t, y2_t, x0, y0;
  real a11, a12, a22;
  size_t num_new = 0;
  bool fe_left, fe_right;

  ne = EL->NumEntries();
  for (i_element = EL->First(); 
       i_element < ne; 
       i_element = EL->Next(i_element)) {
    if ((EL->NumNodes(i_element) == 3) && MESH->ElementMarked(i_element)) {
      for(i_node = 0; i_node < 3; i_node++) {
	if (i_node == 0) p1 = EL->Node(i_element, 2);
	else p1 = EL->Node(i_element, i_node - 1);
	p2 = EL->Node(i_element, i_node);
	x1_t = PL->x(p1);
	y1_t = PL->y(p1);
	x2_t = PL->x(p2);
	y2_t = PL->y(p2);
	x0 = .5*(x1_t + x2_t);
	y0 = .5*(y1_t + y2_t);
	a11 = .5 * (MESH->A11(p1) + MESH->A11(p2));
	a12 = .5 * (MESH->A12(p1) + MESH->A12(p2));
	a22 = .5 * (MESH->A22(p1) + MESH->A22(p2));
	MESH->Transform(x0, y0, a11, a12, a22, x1_t, y1_t);
	MESH->Transform(x0, y0, a11, a12, a22, x2_t, y2_t);
	nd = .5*(MESH->ND(p1) + MESH->ND(p2));
	length = nd * DIST(x1_t, y1_t, x2_t, y2_t);
	if (length > l_max) {
	  MESH->FindElements(p1, p2, e_left, e_right, fe_left, fe_right);
	  if (fe_left && fe_right) {
	    if (e_left != i_element) break;//Error("point <-> element connection damaged");
	    if (EL->NumNodes(e_right) == 3) {
	      n = MESH->FindNode(e_left, p1);
	      pA = EL->PermNode(e_left, n + 2);
	      pB = p1;
	      n = MESH->FindNode(e_right, p1);
	      pC = EL->PermNode(e_right, n + 1);
	      pD = p2;
	      PEL->DeleteFrom(e_left);
	      PEL->DeleteFrom(e_right);
	      EL->DelEntry(e_left);
	      EL->DelEntry(e_right);
	      p_new = PL->AddEntry();
	      PL->x(p_new) = .5*(PL->x(pB) + PL->x(pD));
	      PL->y(p_new) = .5*(PL->y(pB) + PL->y(pD));
	      MESH->SetNetDensity(p_new, .5*(MESH->ND(pB) + MESH->ND(pD)));
	      MESH->SetMatrixComp(p_new, 
				  .5*(MESH->A11(pB) + MESH->A11(pD)),
				  .5*(MESH->A12(pB) + MESH->A12(pD)),
				  .5*(MESH->A22(pB) + MESH->A22(pD)));
	      if (VL != NULL) {
		for (ivar = 0; ivar < VL->NumVars(); ivar++) {
		  var[ivar][p_new] = .5*(var[ivar][pB] + var[ivar][pD]);
		};
	      };
	      PL->code(p_new) = 0;
	      e_new = EL->AddElement(3, p_new, pD, pA);
	      PEL->AddTo(p_new, e_new);
	      PEL->AddTo(pD, e_new);
	      PEL->AddTo(pA, e_new);
	      e_new = EL->AddElement(3, p_new, pA, pB);
	      PEL->AddTo(p_new, e_new);
	      PEL->AddTo(pA, e_new);
	      PEL->AddTo(pB, e_new);
	      e_new = EL->AddElement(3, p_new, pB, pC);
	      PEL->AddTo(p_new, e_new);
	      PEL->AddTo(pB, e_new);
	      PEL->AddTo(pC, e_new);
	      e_new = EL->AddElement(3, p_new, pC, pD);
	      PEL->AddTo(p_new, e_new);
	      PEL->AddTo(pC, e_new);
	      PEL->AddTo(pD, e_new);
	      num_new++;
	      break;
	    };
	  }
	};
      }
    };
  };
  cout << num_new << " points added." << endl;
};







