// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TRIANGULATOR2D_HH)
#define TRIANGULATOR2D_HH

class Triangulator2D;

#include <Mesh2DAction.hh>

/**
   Tool to triangulate a given set of points.
*/
class Triangulator2D : public Mesh2DAction
{
protected:
  /** */ bool *usable;
  /** */ bool *in_front;
  /** */ bool *once_in_front;
  /** */ TList<size_t> *front_p1;
  /** */ TList<size_t> *front_p2;
  /** */ PointList *PL;

  /** 
      Generate a triangle using a front-edge.
      @param front the index of the front-edge 
      @return flag if succeeded
  */
  bool GenerateTriangle (size_t front);

  /** 
      Find the best point to form a triangle on a given front-edge (Delaunay).
      FindBest will throw a NotFound_error if it not able to find a point.
      @parameter front the index of the front-edge
      @return the index of the best point 
  */
  size_t FindBest (size_t front);

  /** 
      Find a front in the lists and delete it if found.
      EliminateFront will throw a NotFound_error if it does not find the front.
      @param p1 the first point of the front
      @param p2 the second point of the front
      @return the index of the front
  */
  size_t EliminateFront (size_t p1, size_t p2);

public:
  /** */ Triangulator2D(ConstructArg arg);
  /** */ void InitFronts();
  /** */ void AddFront(size_t p1, size_t p2);
  /** */ virtual void Act();
};

#endif







