// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(VARLIST_HH)
#define VARLIST_HH

#include <mouse_config.h>

class VarList;

#include "MeshDataList.hh"
#include <TMDimList.hh>
#include <TSDimVar.hh>
#include <TMDimVar.hh>
#include <BMap.hh>
#include "mouse_types.hh"
#include "PointList.hh"
#include "VarContainer.hh"

/**
   VarList provides the basic fields and methods to create a set of variables.
   Basicaly it is a VarContainer, which is linked to the mesh's PointList.
   {\bf syntax:}\\
   \begin{verbatim}
   VarList { 
     <--
     MStringList vars
   }
   \end{verbatim}
*/
class VarList : public VarContainer
{
protected: 

#ifdef WITH_VAR3
  /** */ var3_t var;
#endif
  /** */ PointList *PL;

public:
  /** constructor */
  VarList (ConstructArg arg);

  /** deprecated */ 
  int FieldNumber(string name) {
    size_t i = 0, n;
    bool found = false;
    while((i < varlist->NumSubIndices(0)) && (n < 0)) {
      if (varlist->SubIndex(0, i) == name) {
	n = i;
	found = true;
      };
      i++;
    };
    if (!found) Error("couldn't find field: " + name);
    return n;
  };

  /** deprecated */ 
  int VarIndex(string name) {
    size_t i = 0, n;
    bool found = false;
    while((i < varlist->NumSubIndices(1)) && (n < 0)) {
      if (varlist->SubIndex(1, i) == name) {
	n = i;
	found = true;
      };
      i++;
    };
    if (!found) Error("couldn't find variable: " + name);
    return n;
  };

#ifdef WITH_VAR3
  /** deprecated */ 
  real& Var(const int &i, const int &j, const size_t &k) {
    return var[i][j][k];
  };
#endif
};

#endif














