// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "WriteMaximalDiff.hh"

WriteMaximalDiff::WriteMaximalDiff(carg_t arg) : Action(arg)
{
  AddClassName();
  VL = FindObject<VarList>(false, true);
  string name1 = arg.P->ReadString();
  string name2 = arg.P->ReadString();
  var1 = VL->Get2DVar(name1);
  var2 = VL->Get2DVar(name2);
  string filename = arg.P->ReadString();
  CFLM = ResolveVar<CflManager>(arg);
  s = new ofstream(filename.c_str());
};

WriteMaximalDiff::~WriteMaximalDiff()
{
  delete s;
};

void WriteMaximalDiff::Act()
{
  real diff;
  TList<real> maxi(VL->NumVars(), 1, 0.0);
  for (uint iv = 0; iv < VL->NumVars(); iv++) maxi.NewEntry() = 0.0;
  FORALL(i, VL->) {
    for (uint iv = 0; iv < VL->NumVars(); iv++) {
      diff = fabs(var1[iv][i] - var2[iv][i]);
      diff /= CFLM->TimeStep(i);
      if (diff > maxi[iv]) maxi[iv] = diff;
    };
  };
  for (uint iv = 0; iv < VL->NumVars(); iv++) {
    *s << maxi[iv] << " ";
  };
  *s << endl;
};



