// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Thu May 6 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "WriteMeshForSINUS.hh"
#include <iostream.h>
#include <fstream.h>

WriteMeshForSINUS::WriteMeshForSINUS(carg_t arg)
  : Action(arg)
{
  AddClassName();
  filename = arg.P->ReadString();
  filename += ".sinus";
  PL = FindObject<PointList>(false, true);
  EL = FindObject<ElementList>(false, true);
};

void WriteMeshForSINUS::Act()
{
  ofstream s(filename.c_str());
  s << PL->NumActiveEntries() << " " << EL->NumActiveEntries() << endl;
  FORALL(i, PL->) s << PL->x(i) << " " << PL->y(i) << endl;
  FORALL(i, EL->) {
    size_t N = EL->NumNodes(i);
    if (N != 3) Error("'SINUS' can only handle triangles");
    for (size_t j=0; j < N; j++) s << EL->Node(i,j)+1 << " ";
    s << endl;
  };
  FORALL(i, PL->) s << PL->code(i) << endl;
};

WriteMeshForSINUS::~WriteMeshForSINUS()
{
};

