// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "geometry2D.hh"

//
//.. Angle
//
double Angle (double ax, double ay, double bx, double by) {
  // vector (ax,ay) turned right by 90deg
  double nax = ay, nay = -ax;
  // absolute values of vectors (ax,ay) and (bx,by)
  double abs_a = sqrt (SQR(ax) + SQR(ay));
  if (abs_a == 0) return 0.0;
  double abs_b = sqrt (SQR(bx) + SQR(by));
  if (abs_b == 0) return 0.0;
  ax /= abs_a;
  ay /= abs_a;
  bx /= abs_b;
  by /= abs_b;
  abs_a = 1;
  abs_b = 1;
  double cos_alpha = (ax*bx + ay*by) / (abs_a * abs_b);
  if (nax*bx + nay*by >= 0) return -acos (cos_alpha);
  else return acos (cos_alpha);
};

//
//.. Intersection
//
int Intersection (double& k, double &l,
		  double rx1, double ry1,
		  double ux1, double uy1,
		  double rx2, double ry2,
		  double ux2, double uy2)
{
  double DET = (ux1*uy2-uy1*ux2);
  if (fabs(DET) > 1e-6) {
    k = -(ux2*ry2-ux2*ry1-rx2*uy2+rx1*uy2)/DET;
    l = -(-uy1*rx2+ux1*ry2-ux1*ry1+uy1*rx1)/DET;
    return 1;
  } else {
    return 0;
  };
};

//
//.. CircumCircle
//
void CircumCircle (double &xm, double &ym, double &r,
		   double x1, double y1,
		   double x2, double y2,
		   double x3, double y3)
{
  double r1x = .5*(x1 + x2);
  double r1y = .5*(y1 + y2);
  double r2x = .5*(x1 + x3);
  double r2y = .5*(y1 + y3);
  double u1x = y2 - y1;
  double u1y = -(x2 - x1);
  double u2x = y3 - y1;
  double u2y = -(x3 - x1);
  double k, l;
  Intersection (k, l, r1x, r1y, u1x, u1y, r2x, r2y, u2x, u2y);
  xm = r1x + k*u1x;
  ym = r1y + k*u1y;
  r = DIST(x1, y1, xm, ym);
};


//
//.. ComputeNadir
//
void ComputeNadir (double &x1, double &y1,
		   double &x2, double &y2,
		   double &xp, double &yp,
		   double &xn, double &yn, double ls)
{
  double l, vx, vy, sx, sy;
  vx = x2 - x1;
  vy = y2 - y1;
  l = VABS2(vx, vy);
  vx /= l;
  vy /= l;
  sx = xp - x1;
  sy = yp - y1;
  ls = sx*vx + sy*vy;
  ls /= l;
  xn = x1 + ls*(x2-x1);
  yn = y1 + ls*(y2-y1);
};







