// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed Sep 1 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "AnisoAggloMesh2D.hh"

#ifndef AnisoAggloMesh2D_CC
#define AnisoAggloMesh2D_CC

template <class POP>
AnisoAggloMesh2D<POP>::AnisoAggloMesh2D(carg_t arg)
  : AggloMesh2DBasic<POP>(arg)
{
  AddClassName();
  crit_value = arg.P->ReadReal();
  Agglomerate();
  Setup();
};

template <class POP>
AnisoAggloMesh2D<POP>::~AnisoAggloMesh2D()
{
};

template<class POP>
void AnisoAggloMesh2D<POP>::Agglomerate()
{
  TList<bool> marked(PL_F,false);
  size_t done = 0; // the number of nodes agglomerated in current level

  // going through all the nodes to agglomerate
  size_t cur_node = PL_F->begin();
  size_t limit = 0;
  FORALL(i, PL_F->) if (!mg_agent->IsHiddenNode(i)) limit++;
  while (done < limit) {

    // a node to agglomerate ?
    if (!marked[cur_node] && !mg_agent->IsHiddenNode(cur_node)) {

      // check if this node is potentially "standalone"
      size_t nb_neighbour = POPL_F->NumFLinks(cur_node);
      size_t nb_free_neighbours = 0;
      for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
        size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
	if (!marked[neighbour] && !mg_agent->IsHiddenNode(neighbour)) {
	  nb_free_neighbours++;
	};
      };

      bool node_agglomerated = false;

      if (nb_free_neighbours > 0) {
	// node is ok => agglomerate it
	
	// figure out if this points is in an anisotropic region
	real d_min, d_max;
	bool first = true;
	size_t min_d_node;
	for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
	  size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
	  real d = DIST(PL_F->x(cur_node), PL_F->y(cur_node),
			PL_F->x(neighbour), PL_F->y(neighbour));
	  if (first) {
	    first = false;
	    d_min = d;
	    d_max = d;
	    min_d_node = neighbour;
	  } else {
	    if (d < d_min) {
	      d_min = d;
	      min_d_node = neighbour;
	    };
	    if (d > d_max) d_max = d;
	  };
	};
	bool is_aniso = d_max/d_min > crit_value;

	if (!is_aniso || (!marked[min_d_node] && !mg_agent->IsHiddenNode(min_d_node))) {

	  // it is possible to create an agglomerate with this node 'cur_node'
	  marked[cur_node] = true;
	  node_agglomerated = true;

	  // create a new coarse node
	  size_t cur_agglo = PL->AddEntry();
	  PL->SetIVol(cur_agglo, PL_F->Vol(cur_node));
	  
	  // this is bogus - will be changed !!!
	  PL->x(cur_agglo) = PL_F->x(cur_node);
	  PL->y(cur_agglo) = PL_F->y(cur_node);
	  PL->code(cur_agglo) = 0;
	  
	  // set the fine -> coarse relation
	  mg_agent->SetCoarsePoint(cur_node, cur_agglo);
	  //size_t agglo_size = done;
	  done++;
	  
	  if (!is_aniso) {
	    // agglomerate all free neighbours
	    //size_t nb_neighbour = POPL_F->NumFLinks(cur_node);
	    for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
	      size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
	      if (!marked[neighbour] && !mg_agent->IsHiddenNode(neighbour)) {
		marked[neighbour] = true;
		
		// a free node, get it !
		mg_agent->SetCoarsePoint(neighbour, cur_agglo);
		real vol = PL->IVol(cur_agglo);
		vol += PL_F->Vol(neighbour);
		PL->SetIVol(cur_agglo, vol);
		
		// this is bogus - will be changed !!!
		if (PL_F->code(neighbour) != 0) {
		  PL->x(cur_agglo) = PL_F->x(neighbour);
		  PL->y(cur_agglo) = PL_F->y(neighbour);
		  PL->code(cur_agglo) = 0;
		};
		
		done++;
	      };
	    };
	  } else {
	    marked[min_d_node] = true;

	    // a free node, get it !
	    mg_agent->SetCoarsePoint(min_d_node, cur_agglo);
	    real vol = PL->IVol(cur_agglo);
	    vol += PL_F->Vol(min_d_node);
	    PL->SetIVol(cur_agglo, vol);
	    
	    // this is bogus - will be changed !!!
	    if (PL_F->code(min_d_node) != 0) {
	      PL->x(cur_agglo) = PL_F->x(min_d_node);
	      PL->y(cur_agglo) = PL_F->y(min_d_node);
	      PL->code(cur_agglo) = 0;
	    };
	    
	    done++;
	  };
	};
	//agglo_size -= done;
      };

      if (!node_agglomerated) {
	// node would be "standalone"
	// simply put this node in one of the neighbouring agglomerates
	marked[cur_node] = true;
	
	// find closest agglomerate
	bool first = true;
	real d_min;
	size_t d_min_node;
	for (size_t cur_neigh = 0; cur_neigh < nb_neighbour; cur_neigh++) {
	  size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_neigh);
	  if (marked[neighbour]) {
	    real x_agglo = PL->x(mg_agent->CoarsePoint(neighbour));
	    real y_agglo = PL->y(mg_agent->CoarsePoint(neighbour));
	    real d = DIST(PL_F->x(cur_node), PL_F->y(cur_node), x_agglo, y_agglo);
	    if (first) {
	      first = false;
	      d_min = d;
	      d_min_node = neighbour;
	    } else {
	      if (d < d_min) {
		d_min = d;
		d_min_node = neighbour;
	      };
	    };
	  };
	};
	size_t cur_agglo = mg_agent->CoarsePoint(d_min_node);
	mg_agent->SetCoarsePoint(cur_node, cur_agglo);
	real vol = PL->IVol(cur_agglo);
	vol += PL_F->Vol(cur_node);
	PL->SetIVol(cur_agglo, vol);
	done++;
      };
    };
    // restart with another node !!! stupid choice can be better
    cur_node = PL_F->next(cur_node);
  };

  // inverse the volume
  FORALL(i, PL->) PL->SetIVol(i, PL->Vol(i));
};


#endif

