// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#ifndef CalcGradGreen2D_HH
#define CalcGradGreen2D_HH

template<class POP> class CalcGradGreen2D;

#include <CalcGrad2D.hh>
#include <TFEOperation2D.hh>
#include <TBEOperation2D.hh>
#include <TBPOperation.hh>

/** */
template<class POP>
class CalcGradGreen2D : public CalcGrad2D<POP>
{
protected:
  /** */
  class EvalGreenInt : public CalcGrad2D<POP>::Grad2DOperator
  {
  protected:
    /** */ real var_av;
  public:
    /** */ EvalGreenInt(carg_t arg);
    /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
			real nx, real ny, bool mod_from, bool mod_to);
  };

  /** */
  class EvalBoundaryGreenInt : public CalcGrad2D<POP>::Grad2DOperator
  {
  protected:
    /** */ real var_av;
  public:
    /** */ EvalBoundaryGreenInt(carg_t arg);
    /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
			real nx, real ny, bool mod_from, bool mod_to);
  };

  /** */
  class CorrectBoundaryGrad : public CalcGrad2D<POP>::Grad2DOperator
  {
  protected:
    /** */ POP *POPL;
    
  public:
    /** */ CorrectBoundaryGrad(carg_t arg);
    /** */ void operate(size_t i_tmp, size_t i);
  };

  /** */ Action *eval_green_int;
  /** */ TBPOperation<CorrectBoundaryGrad,POP> *correct_boundaries;
  /** */ TBEOperation2D<EvalBoundaryGreenInt,POP> *eval_b_green_int;
  /** */ void EvalInt() { eval_green_int->Act(); eval_b_green_int->Act(); }; 
  /** */ void CorrectBoundaries() { correct_boundaries->Act(); }

public:
  /** */ CalcGradGreen2D(carg_t arg);
  /** */ virtual void Act();
};

template<class POP>
inline void CalcGradGreen2D<POP>::EvalGreenInt::operate(size_t ie, size_t ip, 
							size_t from, size_t to, 
							real nx, real ny, 
							bool mod_from, bool mod_to)
{
  for (iv = 0; iv < VL->NumVars(); iv++) {
    var_av = .5*(var[iv][from] + var[iv][to]);
    if (mod_from) {
      gx[iv][from] += var_av * nx;
      gy[iv][from] += var_av * ny;
    };
    if (mod_to){
      gx[iv][to] -= var_av * nx;
      gy[iv][to] -= var_av * ny;
    };
  };
};

template<class POP>
inline void CalcGradGreen2D<POP>::EvalBoundaryGreenInt::operate(size_t ie, size_t ip, 
								size_t from, size_t to, 
								real nx, real ny, 
								bool mod_from, bool mod_to)
{
  for (iv = 0; iv < VL->NumVars(); iv++) {
    //var_av = .5*(var[iv][from] + var[iv][to]);
    if (mod_from) {
      gx[iv][from] -= var[iv][from] * nx;
      gy[iv][from] -= var[iv][from] * ny;
    };
    if (mod_to){
      gx[iv][to] -= var[iv][to] * nx;
      gy[iv][to] -= var[iv][to] * ny;
    };
  };
};

template<class POP>
inline void CalcGradGreen2D<POP>::CorrectBoundaryGrad::operate(size_t i_tmp, size_t i)
{
  real Cxx = 0;
  real Cxy = 0;
  real Cyy = 0;
  real xp = PL->x(i);
  real yp = PL->y(i);
  size_t N = POPL->NumFLinks(i);
  for (size_t j = 0; j < N; j++) {
    size_t pj = POPL->FLinkPoint(i,j);
    real xj = PL->x(pj);
    real yj = PL->y(pj);
    Cxx += 2*(xp-xj)*(xp-xj);
    Cxy += 2*(yp-yj)*(xp-xj);
    Cyy += 2*(yp-yj)*(yp-yj);
  };
  real ID = 1.0/(Cxx*Cyy-Cxy*Cxy);
  for (size_t v = 0; v < VL->NumVars(); v++) {
    real Cx = 0;
    real Cy = 0;
    for (size_t j = 0; j < N; j++) {
      size_t pj = POPL->FLinkPoint(i,j);
      real xj = PL->x(pj);
      real yj = PL->y(pj);
      Cx += 2*(var[v][pj]*(xp-xj)+var[v][i]*(xj-xp));
      Cy += 2*(var[v][pj]*(yp-yj)+var[v][i]*(yj-yp));
    };
    gx[v][i] = ID*(Cxy*Cy-Cx*Cyy);
    gy[v][i] = -ID*(Cxx*Cy-Cxy*Cx);
  };
};

#include "CalcGradGreen2D.cc"

#endif







