#ifndef CoarsenMesh2D_HH
#define CoarsenMesh2D_HH

template<class POP> class CoarsenMesh2D;

#include <MObject.hh>
#include <MultiGridAgent.hh>

/**
   A general base for methods to coarsen a grid. This is generally 
   used for multi grid algorithms. This Action sets up the necessary
   links between the fine and coarse mesh. It is also used by a
   MultiGridAgent to link all agents together, to obtain access to
   the finest as well as the coarsest level.
*/
template <class POP>
class CoarsenMesh2D : public MObject
{
protected:
  /** the point information of the aggregated mesh */ 
  PointList *PL;

  /** the boundary point information of the aggregated mesh */ 
  BoundaryPointList *BPL;

  /** the Pop information of the aggregated mesh */ 
  POP *POPL;

  /** the point information of the fine mesh */ 
  PointList *PL_F;

  /** the boundary point information of the fine mesh */ 
  BoundaryPointList *BPL_F;

  /** the Pop information of the fine mesh */ 
  POP *POPL_F;

  /**  */ 
  MultiGridAgent<POP> *mg_agent;

public:
  /** */ CoarsenMesh2D(carg_t arg);

  /**
     return a pointer to the MultiGridAgent of the next finer grid level
     @return the pointer
  */
  MultiGridAgent<POP>* FineMGA() { return mg_agent; };
};

#include "CoarsenMesh2D.cc"

#endif
