// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri Aug 20 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef ComputeCoarseGridBoundaryGrad2D_HH
#define ComputeCoarseGridBoundaryGrad2D_HH

#include <mouse_config.h>

template <class POP> class ComputeCoarseGridBoundaryGrad2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <CoarseGridBoundaryOperator.hh>
#include <BoundaryPointList.hh>

/**
*/
template <class POP>
class ComputeCoarseGridBoundaryGrad2D : public CoarseGridBoundaryOperator<POP>
{
protected:
  /** */ POP *POPL;
  /** */ BoundaryPointList *BPL;
  /** */ VarContainer::var2_t dv_dx;
  /** */ VarContainer::var2_t dv_dy;

public:
  /** */ ComputeCoarseGridBoundaryGrad2D (carg_t arg);
  /** */ virtual ~ComputeCoarseGridBoundaryGrad2D ();
  /** */ void operate(size_t i_tmp, size_t i);
};

template <class POP>
inline void ComputeCoarseGridBoundaryGrad2D<POP>::operate(size_t p_tmp, size_t p)
{
  if (!mga->IsHiddenNode(p)) {
    
    // compute an estimate for the boundary length
    real l = 0;
    TList<size_t>& bpoints = *mga->BoundaryPoints(p);
    FORALL(i, bpoints.) {
      size_t bp = bpoints[i];
      size_t N = POPL->NumBLinks(bp);
      for (size_t j = 0; j < N; j++) {
	l += sqrt(SQR(POPL->Bnx(bp, j)) + SQR(POPL->Bny(bp, j)));
      };
    };
    l = .5*PL->Vol(p)/l;

    // compute a gradient estimation in normal direction
    FORALL(i, bpoints.) {
      size_t bp = bpoints[i];
      for (size_t iv = 0; iv < VL->NumVars(); iv++) {
	real delta = (var[iv][p] - var[iv][bp]) / l;
	dv_dx[iv][bp] = delta * BPL->Nx(bp);
	dv_dy[iv][bp] = delta * BPL->Ny(bp);
      };
    };
  };
};

#include "ComputeCoarseGridBoundaryGrad2D.cc"

#endif


