// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue Aug 24 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "ExtractBoundary2D.hh"

#ifndef ExtractBoundary2D_CC
#define ExtractBoundary2D_CC

template <class POP>
ExtractBoundary2D<POP>::ExtractBoundary2D(carg_t arg)
  : Action(arg)
{
  AddClassName();
  POPL = FindObject<POP>(false, true);
  PL = FindObject<PointList>(false, true);
  start_point = arg.P->ReadIndex();
  filename = arg.P->ReadString();
};

template <class POP>
ExtractBoundary2D<POP>::~ExtractBoundary2D()
{
};

template <class POP>
void ExtractBoundary2D<POP>::Act()
{
  TList<size_t> points(100, 100);
  size_t p = start_point;
  if (PL->code(p) == 0) Error("This is not a boundary node.");
  do {
    points.NewEntry() = p;
    if (POPL->NumBLinks(p) != 2) {
      Error("Boundary is damaged.");
    };
    size_t p1 = POPL->BLinkPoint(p, 0);
    size_t p2 = POPL->BLinkPoint(p, 1);
    real vx1 = PL->x(p1) - PL->x(p);
    real vy1 = PL->y(p1) - PL->y(p);
    real nx1 = POPL->Bnx(p, 0);
    real ny1 = POPL->Bny(p, 0);
    TurnLeft(vx1, vy1);
    if (vx1*nx1 + vy1*ny1 > 0) p = p1;
    else p = p2;
  } while (p != start_point);
  ofstream f(filename.c_str());
  f << points.NumEntries() << endl;
  FORALL(i, points.) {
    f << PL->x(points[i]) << ' ' 
      << PL->y(points[i]) << ' ' 
      << PL->code(points[i]) << endl;
  };
};

#endif

