// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Mon Aug 30 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef IncompressiblePressureExtrapolator2D_HH
#define IncompressiblePressureExtrapolator2D_HH

#include <mouse_config.h>

template <class POP> class IncompressiblePressureExtrapolator2D;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <IncompressibleOperator2D.hh>
#include <BoundaryPointList.hh>
#include <geometry2D.hh>

/**
*/
template <class POP>
class IncompressiblePressureExtrapolator2D : public IncompressibleOperator2D
{
protected:
  /** */ POP *POPL;
  /** */ BoundaryPointList *BPL;

public:
  /** */ IncompressiblePressureExtrapolator2D (carg_t arg);
  /** */ virtual ~IncompressiblePressureExtrapolator2D ();
  /** */ void operate(size_t i_tmp, size_t i);
};

template <class POP>
inline void IncompressiblePressureExtrapolator2D<POP>::operate(size_t i_tmp, size_t i)
{
  real nx = BPL->Nx(i);
  real ny = BPL->Ny(i);
  
  // find neighbouring point which lies most in the direction [nx,ny]
  real s = -1;
  size_t i_extra;

  // loop over all neighbours
  size_t nn = POPL->NumFLinks(i);
  for (size_t n = 0; n < nn; n++) {
    size_t np = POPL->FLinkPoint(i, n);
    real vx = PL->x(np) - PL->x(i);
    real vy = PL->y(np) - PL->y(i);
    real v_norm = DIST(0, 0, vx, vy);
    vx /= v_norm;
    vy /= v_norm;
    real sn = vx*nx + vy*ny;
    if (sn > s) {
      s = sn;
      i_extra = np;
    };
  };

  // set pressure to value of this point
  p[i] = p[i_extra];

  // cross the fingers now !!!!
};

#include "IncompressiblePressureExtrapolator2D.cc"

#endif
