// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 5 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#include "OptimAggloMesh2D.hh"
#ifndef OptimAggloMesh2D_CC
#define OptimAggloMesh2D_CC

#include <AggloShape2D.hh>

template <class POP>
OptimAggloMesh2D<POP>::OptimAggloMesh2D(ConstructArg arg)
  : AggloMesh2DBasic<POP>(arg)
{
  AddClassName();
  w_iso = arg.P->ReadReal();
  e_iso = arg.P->ReadReal();
  w_shape = arg.P->ReadReal();
  e_shape = arg.P->ReadReal();
  w_size = arg.P->ReadReal();
  e_size = arg.P->ReadReal();
  fine_points = new TList<TList<size_t>* >(PL, NULL);
  marked = new TList<bool> (PL_F, false);
  modified =  new TList<bool> (PL_F, true);
  shape = new AggloShape2D<POP>(mg_agent);
  FORALL(i, fine_points->) fine_points->At(i) = new TList<size_t>(6, 2);
  Agglomerate();
  FORALL(i, PL_F->) {
    if (!mg_agent->IsHiddenNode(i)) {
      size_t ic = mg_agent->CoarsePoint(i);
      if (fine_points->At(ic) == NULL) fine_points->At(ic) = new TList<size_t>(6, 2);
      fine_points->At(ic)->NewEntry() = i;
    };
  };
  cout << "\noptimizing level " << mg_agent->Level() + 1 << endl;
  numrax = 1;
  uint numrax_count = 0;
  while (Optimize()) {
    numrax_count++;
    if (numrax_count > 10) numrax *= 1.1;
  };

  /*
  {
    ofstream f("crit.dat");
    FORALL(i, PL->) {
      f << ShapeCrit(fine_points->At(i)) << " "
	<< IsotropyCrit(fine_points->At(i)) << " "
	<< SizeCrit(fine_points->At(i)) << endl;
    };
  };
  */
  FORALL(i, fine_points->) delete fine_points->At(i);
  delete fine_points;
  delete marked;
  delete modified;
  Setup();
};

template<class POP>
void OptimAggloMesh2D<POP>::Agglomerate()
{
  TList<bool> marked(PL_F,false);
  size_t done = 0; // the number of nodes agglomerated in current level

  // going through all the nodes to agglomerate
  size_t cur_node = PL_F->begin();
  size_t limit = 0;
  FORALL(i, PL_F->) if (!mg_agent->IsHiddenNode(i)) limit++;
  while (done < limit) {

    // a node to agglomerate ?
    if (!marked[cur_node] && !mg_agent->IsHiddenNode(cur_node)) {

      // check if this node is potentially "standalone"
      size_t nb_neighbour = POPL_F->NumFLinks(cur_node);
      size_t nb_free_neighbours = 0;
      for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
        size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
	if (!marked[neighbour]) nb_free_neighbours++;
      };
      if (nb_free_neighbours > 0) {
	// node is ok => agglomerate it
	
	marked[cur_node] = true;

	// create a new coarse node
	size_t cur_agglo = PL->AddEntry();
	PL->SetIVol(cur_agglo, PL_F->Vol(cur_node));
	
	// this is bogus - will be changed !!!
	PL->x(cur_agglo) = PL_F->x(cur_node);
	PL->y(cur_agglo) = PL_F->y(cur_node);
	PL->code(cur_agglo) = 0;
	
	// set the fine -> coarse relation
	mg_agent->SetCoarsePoint(cur_node, cur_agglo);
	size_t agglo_size = done;
	done++;
	
	// agglomerate all free neighbours
	//size_t nb_neighbour = POPL_F->NumFLinks(cur_node);
	for (size_t cur_nghb = 0; cur_nghb < nb_neighbour; cur_nghb++) {
	  size_t neighbour = POPL_F->FLinkPoint(cur_node, cur_nghb);
	  if (!marked[neighbour] && !mg_agent->IsHiddenNode(neighbour)) {
	    marked[neighbour] = true;
	    
	    // a free node, get it !
	    mg_agent->SetCoarsePoint(neighbour, cur_agglo);
	    real vol = PL->IVol(cur_agglo);
	    vol += PL_F->Vol(neighbour);
	    PL->SetIVol(cur_agglo, vol);
	    
	    // this is bogus - will be changed !!!
	    if (PL_F->code(neighbour) != 0) {
	      PL->x(cur_agglo) = PL_F->x(neighbour);
	      PL->y(cur_agglo) = PL_F->y(neighbour);
	      PL->code(cur_agglo) = 0;
	    };
	    
	    done++;
	  };
	};
	agglo_size -= done;
      } else {
	// node would be "standalone"
	// simply put this node in one of the neighbouring agglomerates
	marked[cur_node] = true;
	size_t neighbour = POPL_F->FLinkPoint(cur_node, 0);
	size_t cur_agglo = mg_agent->CoarsePoint(neighbour);
	mg_agent->SetCoarsePoint(cur_node, cur_agglo);
	real vol = PL->IVol(cur_agglo);
	vol += PL_F->Vol(cur_node);
	PL->SetIVol(cur_agglo, vol);
	done++;
      };
    };
    // restart with another node !!! stupid choice can be better
    cur_node = PL_F->next(cur_node);
  };

  // inverse the volume
  FORALL(i, PL->) PL->SetIVol(i, PL->Vol(i));
};


template <class POP>
real  OptimAggloMesh2D<POP>::AggloVol(TList<size_t> *points)
{
  real vol = 0;
  FORALL(i, points->) vol += PL_F->Vol(points->At(i));
  return vol;
};

template <class POP>
void OptimAggloMesh2D<POP>::CompGravCentre(TList<size_t> *points, real &x_gc, real &y_gc)
{
  x_gc = 0;
  y_gc = 0;
  real vol = 0;
  FORALL(i, points->) {
    x_gc += PL_F->x(points->At(i)) * PL_F->Vol(points->At(i));
    y_gc += PL_F->y(points->At(i)) * PL_F->Vol(points->At(i));
    vol += PL_F->Vol(points->At(i));
  };
  x_gc /= vol;
  y_gc /= vol;
};

template <class POP>
void OptimAggloMesh2D<POP>::ComputeInertia(real dx0, real dy0, 
					   real dx1, real dy1, 
					   real x_gc, real y_gc,
					   TList<size_t> *points, 
					   real &i0, real &i1)
{
  real d0 = DIST(0,0,dx0,dy0);
  real d1 = DIST(0,0,dx1,dy1);
  dx0 /= d0;
  dy0 /= d0;
  dx1 /= d1;
  dy1 /= d1;
  i0 = 0;
  i1 = 0;
  FORALL(i, points->) {
    real rx = PL_F->x(points->At(i)) - x_gc;
    real ry = PL_F->y(points->At(i)) - y_gc;
    real r0 = dx0*rx + dy0*ry;
    real r1 = dx1*rx + dy1*ry;
    real Vi = PL_F->Vol(points->At(i));
    i0 += (fabs(r1) * Vi);
    i1 += (fabs(r0) * Vi);
  };
};


template <class POP>
OptimAggloMesh2D<POP>::~OptimAggloMesh2D()
{
};

template <class POP>
real OptimAggloMesh2D<POP>::ShapeCrit(TList<size_t> *points)
{
  //AggloShape2D<POP> shape(mg_agent, points);
  shape->Set(points);
  real vol = AggloVol(points);
  real vol_appr = shape->RMax() * shape->RMin() * M_PI;
  real crit = vol/vol_appr;

  /*
  real vol = AggloVol(points);
  real L = 0;
  FORALL(i, points->) {
    size_t p1 = points->At(i);
    size_t N = POPL_F->NumFLinks(p1);
    for(size_t j = 0; j < N; j++) {
      size_t p2 = POPL_F->FLinkPoint(p1,j);
      
      // check if the link is within the set 'points'
      try {
	points->FindItem(p2);
      } catch (NotFound_error) {
	// add the field vector if it is not
	L += DIST(0, 0, POPL_F->Fnx(p1,j), POPL_F->Fny(p1,j));
      };
    };

    if (mg_agent != mg_agent->Finest()) {
      // get a list of all boundary points for the field node
      TList<size_t> *b_nodes = mg_agent->BoundaryPoints(p1);
            
      // loop over all this boundary nodes
      if (b_nodes != NULL) {
	FORALL(j, b_nodes->) {
	  size_t pb1 = b_nodes->At(j);
	  size_t N = POPL_F->NumBLinks(pb1);
	  for(size_t k = 0; k < N; k++) {
	    L += DIST(0, 0, POPL_F->Bnx(pb1,k), POPL_F->Bny(pb1,k));
	  };
	};
      };
    } else {
      // simply add all boundary links
      size_t N = POPL_F->NumBLinks(p1);
      for(size_t j = 0; j < N; j++) {
	L += DIST(0, 0, POPL_F->Bnx(p1,j), POPL_F->Bny(p1,j));
      };
    };
  };
  //cout << L << endl;

  real crit = 16*vol/(L*L);
  */
  
  if (crit > 1) crit -= (crit-1);
  return crit;
};

template <class POP>
real OptimAggloMesh2D<POP>::IsotropyCrit(TList<size_t> *points)
{
  if (points->NumActiveEntries() == 1) return 0;

  real x_gc, y_gc;
  CompGravCentre(points, x_gc, y_gc);

  real xi, yi;
  ComputeInertia(1, 0, 0, 1, x_gc, y_gc, points, xi, yi);
  real abs1 = DIST(0, 0, xi, yi);
  //cout << "\n\n\n("<<xi<<","<<yi<<")\n";
    
  real dx = xi/abs1;
  real dy = yi/abs1;
  real xi1, yi1, xi2, yi2;
  ComputeInertia(dx, dy, dy, -dx, x_gc, y_gc, points, xi1, yi1);
  ComputeInertia(-dx, dy, -dy, -dx, x_gc, y_gc, points, xi2, yi2);
  if (yi2 < yi1) {
    xi = xi2;
    yi = yi2;
  } else {
    xi = xi1;
    yi = yi1;
  };
  real abs2 = .5*DIST(0, 0, xi, yi);
  //cout << "("<<xi<<","<<yi<<")\n";
    
  //cout << "[("<<inertia00<<","<<inertia10<<"),("<<inertia01<<","<<inertia11<<")]\n";
  return yi/abs1;
};

template <class POP>
real OptimAggloMesh2D<POP>::SizeCrit(TList<size_t> *points)
{
  real x = points->NumActiveEntries()*.25;
  real f = pow(fabs(1 - x),2.0);
  if (x > 1) f *= .05;
  real N = 1 - f;
  if (N > 0) return N;
  else return 0;
};

template <class POP>
real OptimAggloMesh2D<POP>::SelectCrit(TList<size_t> *points)
{
  real shape = ShapeCrit(points);
  real size = SizeCrit(points);
  real iso = IsotropyCrit(points);
  //return 1*pow(shape,.8) + 0*pow(iso,.8) + 1*pow(size,1.0);
  //return 1*pow(shape,.8) + 1.5*size;
  return w_shape*pow(shape,e_shape) + w_iso*pow(iso,e_iso) + w_size*pow(size,e_size);
  //return size;
};

/*
template <class POP>
real OptimAggloMesh2D<POP>::SelectCrit(TList<size_t> *points)
{
  AggloShape2D<POP> shape(mg_agent, points);
  real vol = AggloVol(points);
  real vol_ave = 4 * vol / points->NumActiveEntries();
  real r_ave = sqrt(vol_ave/M_PI);
  real vol_appr = shape.RMax() * shape.RMin() * M_PI;

  // compute the shape criterion
  real crit_shape;
  real w_shape = 1;
  real e_shape = .8;
  crit_shape = vol/vol_appr;

  // compute the isotropy criterion
  real w_iso = 0;
  real e_iso = 1;
  real crit_iso;
  {
    // compute an approximation for the gravity centre
    real x_gc = 0;
    real y_gc = 0;
    FORALL(i, points->) {
      x_gc += PL_F->x(points->At(i)) * PL_F->Vol(points->At(i));
      y_gc += PL_F->y(points->At(i)) * PL_F->Vol(points->At(i));
    };
    x_gc /= vol;
    y_gc /= vol;

    // compute the inertia in x and y direction
    real xi, yi;
    ComputeInertia(1, 0, 0, 1, x_gc, y_gc, points, xi, yi);
    real abs1 = DIST(0, 0, xi, yi);
    cout << "\n\n\n("<<xi<<","<<yi<<")\n";
    
    // compute the inertia in the system 1/|(xi,yi)| * [(xi,yi),(-yi,xi)]
    real dx = xi/abs1;
    real dy = yi/abs1;
    real xi1, yi1, xi2, yi2;
    ComputeInertia(dx, dy, dy, -dx, x_gc, y_gc, points, xi1, yi1);
    ComputeInertia(-dx, dy, -dy, -dx, x_gc, y_gc, points, xi2, yi2);
    if (yi2 < yi1) {
      xi = xi2;
      yi = yi2;
    } else {
      xi = xi1;
      yi = yi1;
    };
    real abs2 = .5*DIST(0, 0, xi, yi);
    cout << "("<<xi<<","<<yi<<")\n";
    
    //cout << "[("<<inertia00<<","<<inertia10<<"),("<<inertia01<<","<<inertia11<<")]\n";
    crit_iso = yi/abs1;
  };

  // compute the size criterion
  real w_size = 0;
  real e_size = .8;
  real crit_size = fabs(vol-vol_ave)/vol_ave;
  if (crit_size > 1) crit_size = 0;
  else crit_size = 1-crit_size;

  real crit = 
    w_shape*pow(crit_shape,e_shape) +
    w_iso*pow(crit_iso,e_iso) + 
    w_size*pow(crit_size,e_size);
  cout << "\nvol_agglo  = " << vol 
       << "\nvol_appr   = " << vol_appr
       << "\nvol_ave    = " << vol_ave
       << "\nr_min      = " << shape.RMin() 
       << "\nr_max      = " << shape.RMax()
       << "\nalpha_min  = " << shape.AlphaMin()/M_PI*180
       << "\nalpha_max  = " << shape.AlphaMax()/M_PI*180
       << "\ncrit_shape = " << crit_shape
       << "\ncrit_iso   = " << crit_iso
       << "\ncrit_size  = " << crit_size
       << "\ncrit       = " << crit << endl << endl;
  return crit;
};
*/

/*
template<class POP>
void OptimAggloMesh2D<POP>::Agglomerate()
{
  TList<bool> marked(PL_F,false);
  size_t done = 0; // the number of nodes agglomerated in current level
  
  // going through all the nodes to agglomerate
  size_t cur_upper = PL_F->last();
  size_t cur_lower = PL_F->begin();
  size_t cur_node = cur_upper;
  bool upper = true;
  size_t limit = PL_F->NumActiveEntries();
  if (mg_agent != mg_agent->Finest()) limit -= BPL_F->NumActiveEntries();
  while (done < limit) {
    
    // pick up a inital node 
    if (!marked[cur_node] && !mg_agent->IsHiddenNode(cur_node)) {
      
      // declare some working structure
      TList<size_t> ok_points(5,2);
      
      // The first is stand alone, so the best
      size_t i = ok_points.AddEntry();
      ok_points[i] = cur_node; 
      
      // Increase this agglomerate
      for (size_t nbpoint=1; nbpoint<4; nbpoint++){
	TList<size_t> ngb_points(12,4);
	FORALL(j, ok_points.) ngb_points.NewEntry() = ok_points[j];
	
	// initialize
	real bestval = -1e10;
	size_t bestnode;
	size_t k = ok_points.AddEntry();
	
	// compute the effect of add on neighbour
	//FORALL(cur, ok_points.) {
	for(size_t cur = ok_points.begin(); cur < ok_points.last(); cur = ok_points.next(cur)) {
	  size_t cur_ok = ok_points[cur];
	  size_t nb_ngh = POPL_F->NumFLinks(cur_ok);
	  for (size_t nghb = 0; nghb < nb_ngh; nghb++) {
	    size_t p2 = POPL_F->FLinkPoint(cur_ok, nghb);
	    
	    // check if it has been already treated
	    try {
	      ngb_points.FindItem(p2);
	    } catch (NotFound_error) {
	      ngb_points.NewEntry() = p2;
	      
	      if (!marked[p2]){
		
		// Evaluate the shape of the resulting agglomerate
		ok_points[k] = p2; 
				
		// is it the best we ever found yet ?
		real curval  = SelectCrit(&ok_points);
		if (curval > bestval) {
		  bestval = curval;
		  bestnode  = p2;
		};
	      }; 
	    };
	  };
	};
	
	// we can add the best one to the ok_points list
	if (bestval > -1000) {
	  ok_points[k] = bestnode;
	} else {
	  ok_points.DelEntry(k);
	  break;
	};
      };
      
      // This agglomerate is finished, let's marked nodes and build another one
      size_t cur_agglo = PL->AddEntry();
      fine_points->At(cur_agglo) = new TList<size_t>(ok_points.NumEntries(), 2);
      PL->code(cur_agglo) = 0;                // only field coarse nodes !!!
      real vol=0;
      FORALL(i, ok_points.) {
	size_t one_of = ok_points[i];
	marked[one_of] = true;              // mark it  !!!
	mg_agent->SetCoarsePoint(one_of, cur_agglo);  
	fine_points->At(cur_agglo)->NewEntry() = one_of;
	
	vol += PL_F->Vol(one_of);
	
	// set the coordinates
	PL->x(cur_agglo) = PL_F->x(one_of);
	PL->y(cur_agglo) = PL_F->y(one_of);
	
	done++;
      };
      PL->SetIVol(cur_agglo, (1./vol));       // set the inverted volume  !!!
      //exit(-1);
    }; 
    
    // next node non agglomerated in the PL_F list
    if (upper) {
      upper = false;
      cur_lower = PL_F->next(cur_lower);
      cur_node = cur_lower;
    } else {
      upper = true;
      cur_upper = PL_F->prev(cur_upper);
      cur_node = cur_upper;
    };
    //cur_node = PL_F->next(cur_node);
  };
  //FORALL(i, PL_F->) cout << i << " -> " << mg_agent->CoarsePoint(i) << endl;
};
*/


template<class POP>
void OptimAggloMesh2D<POP>::MoveTo(size_t i_fine, size_t i_coarse)
{
  size_t cur_agglo = mg_agent->CoarsePoint(i_fine);
  size_t i = fine_points->At(cur_agglo)->FindItem(i_fine);
  fine_points->At(cur_agglo)->DelEntry(i);
  fine_points->At(i_coarse)->NewEntry() = i_fine;
  mg_agent->SetCoarsePoint(i_fine, i_coarse);

  // update the volume
  PL->SetIVol(cur_agglo, 1/(PL->Vol(cur_agglo) - PL_F->Vol(i_fine)));
  PL->SetIVol(i_coarse, 1/(PL->Vol(i_coarse) + PL_F->Vol(i_fine)));
};

template<class POP>
uint OptimAggloMesh2D<POP>::ExchangeFineNodes(size_t p1, size_t p2)
{
  TList<size_t> *points = fine_points->At(p1);
  uint modi = 0;
  size_t num_points = points->NumActiveEntries();
  if (num_points == 1) return modi;
  size_t start_p = points->At(points->last());
  FORALL(i, points->) {
    size_t i_fine = points->At(i);
    if (i_fine == start_p) start_p = points->At(points->begin());
    if (IsOnBorder(i_fine, p1, p2)) {
      FORALL(j, points->) marked->At(points->At(j)) = false;
      if (TouchedNodes(p1, start_p, i_fine) == num_points-1) {
	real crit_p1_1 = SelectCrit(fine_points->At(p1));
	real crit_p2_1 = SelectCrit(fine_points->At(p2));
	TList<size_t> new_points_p1(10,10);
	FORALL(j, fine_points->At(p1)->) {
	  if (fine_points->At(p1)->At(j) != i_fine) {
	    new_points_p1.NewEntry() = fine_points->At(p1)->At(j);
	  };
	};
	TList<size_t> new_points_p2(10,10);
	FORALL(j, fine_points->At(p2)->) new_points_p2.NewEntry() = fine_points->At(p2)->At(j);
	new_points_p2.NewEntry() = i_fine;
	real crit_p1_2 = SelectCrit(&new_points_p1);
	real crit_p2_2 = SelectCrit(&new_points_p2);
	if ((crit_p1_2 + crit_p2_2) > numrax*(crit_p1_1 + crit_p2_1)) {
	  MoveTo(i_fine, p2);
	  modi++;
	  if (fine_points->At(p1)->NumActiveEntries() == 1) return modi;
	};
      };
    };
  };
  return modi;
};

template <class POP>
bool OptimAggloMesh2D<POP>::IsOnBorder(size_t fine_i, size_t coarse_i1, size_t coarse_i2)
{
  bool rval = false;
  for (size_t nghb = 0; nghb < POPL_F->NumFLinks(fine_i); nghb++) {
    size_t p2 = POPL_F->FLinkPoint(fine_i, nghb);
    if ( mg_agent->CoarsePoint(p2) == coarse_i2) rval =true;
  }
  return rval;
};

template<class POP>
uint OptimAggloMesh2D<POP>::Optimize()
{
  uint modi = 0;
  TList<bool> lately_modified(PL,false);
  FORALL(i, PL_F->) {
    if (!mg_agent->IsHiddenNode(i)) {
      size_t N = POPL_F->NumFLinks(i);
      size_t i_coarse = mg_agent->CoarsePoint(i);
      for (size_t j = 0; j < N; j++) {
	size_t j_coarse = mg_agent->CoarsePoint(POPL_F->FLinkPoint(i,j));
	if ((i_coarse != j_coarse) && (modified->At(i_coarse) || modified->At(j_coarse))) {
	  size_t d_modi = ExchangeFineNodes(i_coarse, j_coarse);
	  if (d_modi != 0) {
	    modi += d_modi;
	    lately_modified[i_coarse] = true;
	    lately_modified[j_coarse] = true;
	  };
	};
      };
    };
  };
  cout << modi << " reconfigurations" << endl;
  {
    //ofstream f("crit.dat");
    //FORALL(i, PL->) f << SelectCrit(fine_points->At(i)) << endl;
  };
  FORALL(i, PL->) modified->At(i) = lately_modified[i];
  return modi;
};

template<class POP>
size_t OptimAggloMesh2D<POP>::TouchedNodes(size_t agglo, size_t p, size_t skip_p)
{
  size_t N = 1;
  size_t num_neigh = POPL_F->NumFLinks(p);
  marked->At(p) = true;
  for (size_t i = 0; i < num_neigh; i++) {
    size_t neigh = POPL_F->FLinkPoint(p, i);
    if ((neigh != skip_p) && !marked->At(neigh) && (mg_agent->CoarsePoint(neigh) == agglo)) {
      N += TouchedNodes(agglo, neigh, skip_p);
    };
  };
  return N;
};


#endif
