// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Wed May 5 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef OptimAggloMesh2D_HH
#define OptimAggloMesh2D_HH

#include <mouse_config.h>

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <AggloMesh2DBasic.hh>
#include <AggloShape2D.hh>

/**
*/
template <class POP>
class OptimAggloMesh2D : public AggloMesh2DBasic<POP>
{
protected:
  /** a list of all fine points per agglomerate */
  TList<TList<size_t>* > *fine_points;

  /** */
  TList<bool> *marked;

  /** */
  TList<bool> *modified;

  /** */
  AggloShape2D<POP> *shape;

  /** */
  real numrax;

  /** */
  real w_iso;

  /** */
  real w_shape;

  /** */
  real w_size;

  /** */
  real e_iso;

  /** */
  real e_shape;
  
  /** */
  real e_size;

  /** construct the agglomerated cells */ 
  void Agglomerate();

  /** selection criterion */
  real SelectCrit(TList<size_t> *points);

  /** exchange all fine nodes which would improve the overall quality */
  uint ExchangeFineNodes(size_t p1, size_t p2);

  /** Is a node on a border of the agglomerate */
  bool IsOnBorder(size_t fine_i, size_t coarse_i1, size_t coarse_i2);

  /** */ 
  void ComputeInertia(real dx0, real dy0, real dx1, real dy1, real x_gc, real y_gc, 
		      TList<size_t> *points, real &i0, real &i1);

  /** */
  real AggloVol(TList<size_t> *points);
  
  /** */
  void CompGravCentre(TList<size_t> *points, real &x_gc, real &y_gc);

  /** */
  real ShapeCrit(TList<size_t> *points);

  /** */
  real IsotropyCrit(TList<size_t> *points);

  /** */
  real SizeCrit(TList<size_t> *points);

  /** */
  size_t TouchedNodes(size_t agglo, size_t p, size_t skip_p);

  /** */
  uint Optimize();

public:
  /** */ OptimAggloMesh2D (carg_t arg);
  /** */ virtual ~OptimAggloMesh2D ();

  /**
     move a fine node to another agglomerated cell
  */
  void MoveTo(size_t i_fine, size_t i_coarse);  
};


#include "OptimAggloMesh2D.cc"

#endif
