// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Fri Jun 11 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef RungeKuttaDts_HH
#define RungeKuttaDts_HH

#include <mouse_config.h>

template <class POP> class RungeKuttaDts;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <Operator.hh>
#include <RungeKutta.hh>
#include <MgAllLevelCflManager.hh>

/**
*/
template <class POP>
class RungeKuttaDts : public RungeKutta<POP>
{
protected:
  /** */ 
  struct DtsPartialStep : public SubOperator {
    RungeKuttaDts<POP> *rkdts;
    MgAllLevelCflManager<POP> *CFLM_IMPL;
    DtsPartialStep(carg_t arg);
    void operate(size_t il, size_t);
  };

  /** */ MgAllLevelCflManager<POP> *CFLM_IMPL;
  /** */ string old_name;
  /** */ bool is_first;
  /** */ POperation *partial_dts_step;
  /** */ virtual void Partial() { partial_dts_step->Act(); };

public:
  /** */ RungeKuttaDts (carg_t arg);
  /** */ virtual ~RungeKuttaDts ();
  /** */ MgAllLevelCflManager<POP> *ImplCfl() { return CFLM_IMPL; };
  /** */ string OldName() { return old_name; };
  /** */ bool IsFirst() { return is_first; };
  /** */ virtual void Act();
};
 
template<class POP>
inline void RungeKuttaDts<POP>::DtsPartialStep::operate(size_t il, size_t i)
{
  real dtau = CFLM->CFL() * CFLM->TimeStep(i);
  real dt = CFLM_IMPL->CFL() * CFLM_IMPL->TimeStep(i);
  real C = tstep->Factor()*PL->IVol(i)*dt*dtau/(dt+dtau);
  for (size_t iv = 0; iv < VL->NumVars(); iv++) {
    var_new[iv][i] = var_old[iv][i] + C*var_res[iv][i];

    /*
    var_new[iv][i] = var_old[iv][i];
    var_new[iv][i] += alpha*lambda*var_new[iv][i];
    var_new[iv][i] += alpha*dtau/V*var_res[iv][i];
    var_new[iv][i] /= 1+alpha*lambda;
    */
  };
};

#include "RungeKuttaDts.cc"

#endif
