// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue May 11 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "SimpleMinMax.hh"

#ifndef SimpleMinMax_CC
#define SimpleMinMax_CC

template <class F>
SimpleMinMax<F>::SimpleMinMax(F *an_f, real x_min, real an_x_max, int N)
{
  f = an_f;
  x_max = an_x_max;
  d_x = (x_max-x_min)/N;
  real xf_min = f->calc(x_min);
  real xf_max = f->calc(x_max);
  if (xf_min > xf_max) {
    f_min = xf_max;
    f_max = xf_min;
    xf_min = x_max;
    xf_max = x_min;
  } else {
    f_max = xf_max;
    f_min = xf_min;
    xf_min = x_min;
    xf_max = x_max;
  };
  for (uint i = 0; i <= N; i++) {
    real fx = f->calc(i*d_x);
    if (fx < f_min) {
      f_min = fx;
      xf_min = i*d_x;
    };
    if (fx > f_max) {
      f_max = fx;
      xf_max = i*d_x;
    };
  };
};


#endif

