// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue May 11 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#include "StoredAggloMesh2D.hh"
#include "CalcChDist2D.hh"

#ifndef StoredAggloMesh2D_CC
#define StoredAggloMesh2D_CC

template <class POP>
StoredAggloMesh2D<POP>::StoredAggloMesh2D(carg_t arg)
  : AggloMesh2DBasic<POP>(arg)
{
  AddClassName();
  filename = arg.P->ReadString();
  Agglomerate();
  Setup();
  //CalcChDist2D<POP> ch(this);
  //ch.Act();
};

template <class POP>
void StoredAggloMesh2D<POP>::Agglomerate()
{
  TList<size_t> I_c(PL_F->NumEntries(),100);
  TList<size_t> I_f(&I_c);
  char word[200];
  strcpy(word, "");
  sprintf(word, "%s.L%u_agglo", filename.c_str(), mg_agent->Level());
  cout << word << endl;
  ifstream f(word);
  size_t N;
  f >> N;
  cout << N << endl;
  for (size_t i = 0; i < N; i++) {
    size_t j = I_c.AddEntry();
    f >> I_f[j] >> I_c[j];
  };
  size_t c_max = 0;
  FORALL(i, I_c.) if (I_c[i] > c_max) c_max = I_c[i];
  for(size_t i = 0; i <= c_max; i++) {
    size_t j = PL->AddEntry();
    PL->code(j) = 0;
  };
  FORALL(i, I_c.) mg_agent->SetCoarsePoint(I_f[i], I_c[i]);

  // compute the volume sizes
  FORALL(i, PL->) PL->SetIVol(i, 0);
  FORALL(i, PL_F->) {
    if (!mg_agent->IsHiddenNode(i)) {
      size_t j = mg_agent->CoarsePoint(i);
      PL->SetIVol(j, PL->IVol(j) + PL_F->Vol(i));
    };
  };
  FORALL(i, PL->) if (PL->code(i) == 0) PL->SetIVol(i, PL->Vol(i));
};

template <class POP>
StoredAggloMesh2D<POP>::~StoredAggloMesh2D()
{
};

#endif

