template<class POP>
StructuredBlock2D<POP>::StructuredBlock2D(carg_t arg) : MObject(arg)
{
  AddClassName();
  PL = FindObject<PointList>(false, true);
  POPL = FindObject<POP>(false, true);
  EL = LookForObject<ElementList>(false, true);
  BPL = FindObject<BoundaryPointList>(false, true);
  MIntList *codes = BuildObject<MIntList>(arg);
//   c_dl = arg.P->ReadInt();
//   c_l = arg.P->ReadInt();
//   c_ul = arg.P->ReadInt();
//   c_u = arg.P->ReadInt();
//   c_ur = arg.P->ReadInt();
//   c_r = arg.P->ReadInt();
//   c_dr = arg.P->ReadInt();
//   c_d = arg.P->ReadInt();
  c_dl = codes->At(0);
  c_l = codes->At(1);
  c_ul = codes->At(2);
  c_u = codes->At(3);
  c_ur = codes->At(4);
  c_r = codes->At(5);
  c_dr = codes->At(6);
  c_d = codes->At(7);
  i_linked = false;
  j_linked = false;
};

template<class POP>
void StructuredBlock2D<POP>::InitBlock(size_t a_imax, size_t a_jmax)
{
  imax = a_imax;
  jmax = a_jmax;
  bool origin_set = false;
  for (size_t j=0; j<jmax; j++) {
    for (size_t i=0; i<imax; i++) {
      size_t p = PL->AddEntry();
      if (!origin_set) {
	origin_set = true;
	origin = p;
      };
      Set(i, j, i*1.0, j*1.0, 0);
    };
  };
  for (size_t i=0; i<imax; i++) {
    Set(i, 0, c_d);
    Set(i, jmax-1, c_u);
  };
  for (size_t j=0; j<jmax; j++) {
    Set(0, j, c_l);
    Set(imax-1, j, c_r);
  };
  Set(0, 0, c_dl);
  Set(0, jmax-1, c_ul);
  Set(imax-1, jmax-1, c_ur);
  Set(imax-1, 0, c_dr);
  BPL->Update();
};

template<class POP>
void StructuredBlock2D<POP>::HandleEvent(Event &E)
{
  MObject::HandleEvent(E);
  if (!E.Done()) {
    if ((E.name == "MListCleaned") && (E.sender == PL)) {
      origin -= PL->Offset(origin);
    };
  };
};

template<class POP>
void StructuredBlock2D<POP>::Set(size_t i, size_t j, real a_x, real a_y)
{
  PL->x(point(i,j)) = a_x;
  PL->y(point(i,j)) = a_y;
};

template<class POP>
void StructuredBlock2D<POP>::Set(size_t i, size_t j, real a_x, real a_y, int a_code)
{
  Set(i, j, a_x, a_y);
  PL->code(point(i,j)) = a_code;
};

template<class POP>
void StructuredBlock2D<POP>::Set(size_t i, size_t j, int a_code)
{
  PL->code(point(i,j)) = a_code;
};

template<class POP>
real StructuredBlock2D<POP>::QuadX(size_t i, size_t j)
{
  real qx = x(i,j) + x(i+1,j) + x(i+1,j+1), x(i+1,j);
  qx *= .25;
  return qx;
};

template<class POP>
real StructuredBlock2D<POP>::QuadY(size_t i, size_t j)
{
  real qy = y(i,j) + y(i+1,j) + y(i+1,j+1), y(i+1,j);
  qy *= .25;
  return qy;
};

template<class POP>
void StructuredBlock2D<POP>::AddQuad(size_t p0, size_t p1, size_t p2, size_t p3)
{
  size_t p[5];
  p[0] = p0;
  p[1] = p1;
  p[2] = p2;
  p[3] = p3;
  p[4] = p0;
  real xm=0;
  real ym=0;
  for (int i=0; i<4; i++) {
    xm += PL->x(p[i]);
    ym += PL->y(p[i]);
  };
  xm *= .25;
  ym *= .25;
  for (int i=0; i<4; i++) {
    size_t p1 = p[i];
    size_t p2 = p[i+1];
    real nx = xm - .5*(PL->x(p1) + PL->x(p2));
    real ny = ym - .5*(PL->y(p1) + PL->y(p2));
    if (p1 < p2) {
      TurnRight(nx,ny);
    } else {
      TurnLeft(nx,ny);
    };
    size_t link;
    try {
      link = POPL->FindFLink(p1, p2);
    } catch (NotFound_error) {
      POPL->AddFLink(p1, p2, 0, 0);
      link = POPL->FindFLink(p1, p2);
    };
    //cout << p1 << "->" << p2 << ":" << link << "," << nx << "," << ny << endl;
    POPL->SetFnx(p1, link, POPL->Fnx(p1, link) + nx);
    POPL->SetFny(p1, link, POPL->Fny(p1, link) + ny);
  };
};

template<class POP>
void StructuredBlock2D<POP>::AddEdgesToPop()
{
  size_t p1, p2;
  real nx, ny;
  for(size_t i = 0; i < imax-1; i++) {
    for(size_t j = 0; j < jmax-1; j++) {
      AddQuad(point(i,j), point(i+1,j), point(i+1,j+1), point(i,j+1));
      if (EL != NULL) {
	EL->AddElement(4, point(i,j), point(i+1,j), point(i+1,j+1), point(i,j+1));
      };
    };
  };
  if (i_linked) {
    for (size_t j = 0; j < jmax-1; j++) {
      AddQuad(point(imax-1,j), point(0,j), point(0,j+1), point(imax-1,j+1));
      if (EL != NULL) {
	EL->AddElement(4, point(imax-1,j), point(0,j), point(0,j+1), point(imax-1,j+1));
      };
    };
  };
  if (j_linked) {
    for(size_t i = 0; i < imax-1; i++) {
      AddQuad(point(i,jmax-1), point(i,0), point(i+1,0), point(i+1,jmax-1));
      if (EL != NULL) {
	EL->AddElement(4, point(i,jmax-1), point(i,0), point(i+1,0), point(i+1,jmax-1));
      };
    };
  };
  BPL->Update();
  if (!j_linked) {
    for(size_t i = 0; i < imax-1; i++) {
      POPL->AddBLink(point(i,0), point(i+1,0));
      POPL->AddBLink(point(i+1,jmax-1), point(i,jmax-1));
    };
    if (i_linked) {
      POPL->AddBLink(point(imax-1,0),point(0,0));
      POPL->AddBLink(point(0,jmax-1),point(imax-1,jmax-1));
    };
  };
  if (!i_linked) {
    for(size_t j = 0; j < jmax-1; j++) {
      POPL->AddBLink(point(0,j+1), point(0,j));
      POPL->AddBLink(point(imax-1,j), point(imax-1,j+1));
    };
    if (j_linked) {
      POPL->AddBLink(point(0,jmax-1),point(0,0));
      POPL->AddBLink(point(imax-1,0),point(imax-1,jmax-1));
    };
  };
  POPL->UpdateMetric();
};

template<class POP>
size_t StructuredBlock2D<POP>::GoUp(size_t i, size_t j)
{
  if (j == jmax-1) {
    if (j_linked) return point(i, 0);
    else throw IllegalPoint_error();
  };
  return point(i, j+1);
};

template<class POP>
size_t StructuredBlock2D<POP>::GoDown(size_t i, size_t j)
{
  if (j == 0) {
    if (j_linked) return point(i, jmax-1);
    else throw IllegalPoint_error();
  };
  return point(i, j-1);
};

template<class POP>
size_t StructuredBlock2D<POP>::GoLeft(size_t i, size_t j)
{
  if (i == 0) {
    if (i_linked) return point(imax-1, j);
    else throw IllegalPoint_error();
  };
  return point(i-1, j);
};

template<class POP>
size_t StructuredBlock2D<POP>::GoRight(size_t i, size_t j)
{
  if (i == imax-1) {
    if (i_linked) return point(0, j);
    else throw IllegalPoint_error();
  };
  return point(i+1, j);
};
