// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

template<class OP, class POP>
TBEdgeCalculator2D<OP, POP>::TBEdgeCalculator2D(ConstructArg arg) 
  : BPointCalculator(arg) 
{
  AddClassName();
  op = new OP(carg_t(this, arg.P->SubParser()));
  mesh = FindObject<POP>(false, true);
};

template<class OP, class POP>
void TBEdgeCalculator2D<OP, POP>::CalcGroup
(int code, TList<code_t> *codes, size_t i_begin, size_t i_end)
{
  size_t i_from;
  size_t from, to;
  uint j;
  uint j_max;
  uint gid_f = thread_id;
  uint gid_t;
  bool mod_to;
  bool calc_edge;
  // it is assumed that the group_id of the from point always belongs
  // to this thread (the PointAction object should take care of that)
  bool mod_from = true;
  size_t group_index = 0;
  // check and eventually extend the temporary point memory
  if (i_end - i_begin > point_tmp->NumEntries()) {
    point_tmp->ExtendBy(i_end - i_begin - point_tmp->NumEntries());
  };
  //for (i_from = i_begin; i_from < i_end; i_from = PL->next(i_from)) {
  for (i_from = i_begin; i_from < i_end; i_from = BPL->next(code, i_from)) {
    from = BPL->Point(code, i_from);
    j_max = mesh->NumBLinks(from);
    // check and eventually extend the temporary edge memory
    if (group_index + j_max > edge_tmp->NumEntries()) {
      edge_tmp->ExtendBy(group_index + j_max - edge_tmp->NumEntries());
    };
    for (j = 0; j < j_max; j++) {
      to = mesh->BLinkPoint(from, j);
      if (codes->Has(PL->code(to))) {

	/*
	  gid_t = mesh->GroupID(to);
	  // changed it to <= instead of >= (280399 Oli)
	  mod_to = ((gid_t == gid_f) && (BPL->Prio(from) <= BPL->Prio(to)));
	  mod_from = true;
	  calc_edge = ((from < to) && mod_to);
	*/

	gid_t = mesh->GroupID(to);
	mod_to = (gid_f == gid_t);
	mod_from = true;
	calc_edge = (from <= to);
	if (!calc_edge && !mod_to) calc_edge = true;
	if (calc_edge) op->operate(group_index, i_from - i_begin, from, to, 
				   mesh->Bnx(from, j), mesh->Bny(from, j),
				   mod_from, mod_to);
	group_index++;
      };
    };	
  };
};







