// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TLIST2D_H)
#define TLIST2D_H

template <class T> TList2D;

#include "lists.hh"

/**
   a twodimensional TList.
*/
template <class T>
class TList2D : public List 
{
protected:
  int dim;

  /** array of the stored values */
  T **value;

  virtual void Extend (long int);
  virtual void CopyEntry (long int source_point, long int dest_point);
  virtual void InitEntry (long int i);
public:
  /** the default value for new entries */
  T default_value;

  /** constructor.
    @param a_max_num_entries the initial maximal number of entries
    @param a_delta_entries the initial increment 
    @param a_dim the 2nd dimension */
  TList2D (long int a_max_num_entries, long int a_delta_entries, int a_dim);

  /** constructor.
    @param a_max_num_entries the initial maximal number of entries
    @param a_delta_entries the initial increment 
    @param a_dim the 2nd dimension
    @param a_default_value the initial value for new entries */
  TList2D (long int a_max_num_entries, long int a_delta_entries, 
	   int a_dim, T a_default_value);

  /** constructor.
    @param a_dim the 2nd dimension
    @param a_master a master for this List */
  TList2D (int a_dim, List *a_master);

  /** constructor.
    @param a_dim the 2nd dimension
    @param a_master a master for this List.
    @param a_default_value the initial value for new entries */
  TList2D (int a_dim, List *a_master, T a_default_value);

  /** destructor. */
  ~TList2D ();

  /** The Value of an entry.
    @param i the 1st index of the entry (the shorter one)
    @param j the 2nd index of the entry
    @return the value of this entry */
  T& At (int i, long int j) { return value[i][j]; };

  /** Get a pointer to the data field */
  T** ValuePointer() { return value; };
};

#include "Tlist2D.cc"

#endif








