// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TMDIMLIST_H)
#define TMDIMLIST_H

template <class TValue, class TIndex, int DIM, class MAP> class TMDimList;

#include "TMDimIndex.hh"
#include "TMappedVar.hh"
#include "TList.hh"
#include "TKeyList.hh"

/**
 */
template<class TValue, class TIndex, int DIM, class MAP>
class TMDimList : public List
{
  friend class TMappedVar<TValue, TIndex, DIM, MAP>;

  //public:
protected:
  /** */ TIndex empty_index_value;
  /** */ TKeyList<TMDimIndex<TIndex>, TValue*> *total_index;
  /** */ TList<TIndex> **sub_index;
  /** */ TValue *root_pointer;
  /** */ void **var_pointer;
  /** */ void AllocNewBlocks();
  /** */ virtual void Extend(size_t delta);
  /** */ virtual void InitEntry (size_t i);
  /** */ TValue default_value;
  /** */ TList<TMappedVar<TValue, TIndex, DIM, MAP>*> *mapped_vars;

public:
  /** */ TMDimList(size_t a_max_num_entries, size_t a_delta_entries,
		   TValue a_default_value, TIndex an_empty_index_value);
  /** */ TMDimList(List *a_master, TValue a_default_value, 
		   TIndex an_empty_index_value);
  /** */ virtual ~TMDimList() { cout << "write TMDimList destructor!!!" << endl; };

  /** */ int LengthOfPointerField() const;
  /** */ void SetPointerField();
  /** */ void CreateTotalIndex(int level, TMDimIndex<TIndex> &I);
  /** */ void AddIndex(int i, TIndex new_index);
  /** */ TList<TMDimIndex<TIndex> >* GetSubIndices(TMDimIndex<TIndex> I) const;
  /** */ virtual void CopyEntry(size_t src, size_t dst);
  /** */ TValue* Resolve(TMDimIndex<TIndex> I);
  /** */ size_t NumBlocks() const { return total_index->NumEntries(); };
  /** */ size_t NumSubIndices(int level) const { return sub_index[level]->NumEntries(); };
  /** */ TIndex SubIndex(int level, int i) { return sub_index[level]->At(i); };
  /** */ bool IndexExists(size_t level, TIndex an_index);
};

#include "TMDimList.cc"

#endif







