// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(TSTACK_H)
#define TSTACK_H

template <class T> class TStack;

template <class T>
struct TStackEntry {
  T *data;
  TStackEntry *next;
};

/**
   a LIFO-storage-class. 
*/
template <class T>
class TStack {
protected:
  /** the top entry of the stack */
  TStackEntry<T> *top;
public:
  /** the number of entries */
  long int count;

  /** constructor. */
  TStack ();

  /** destructor. */
  ~TStack () {}; // @@

  /** Put an entry on the stack.
    @param v the value to put on the stack */
  void Push (T v);

  /** Take the top entry from the stack.
    @return the value got from the stack */
  T Pull ();

  /** Get the top entry, but don't remove it.
    @return the top-value of the stack */
  T& Top () { return *(top->data); };

  /** Duplicate the top entry. */
  void DupTop () { Push(Top()); };
};

#include "TStack.cc"

#endif







