// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik
// Universitaet Duisburg, Germany
// Institute for Combustion and Gas Dynamics
// University of Duisburg, Germany
// Tue May 25 1999
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!
#ifndef TestProjector2D2_HH
#define TestProjector2D2_HH

#include <mouse_config.h>

template<class POP> class TestProjector2D2;

#include <mouse_types.hh>
#include <ConstructArg.hh>
#include <MusclProjector2D2.hh>
#include <geometry2D.hh>
#include <BoundaryPointList.hh>

/**
*/
template <class POP>
class TestProjector2D2 : public MusclProjector2D2
{
protected:
  /** */ POP *POPL;
  /** */ BoundaryPointList *BPL;

public:
  /** */ TestProjector2D2 (carg_t arg);
  /** */ virtual ~TestProjector2D2 ();
  /** */ void operate(size_t ie, size_t ip, size_t from, size_t to, 
		      real nx, real ny, bool mod_from, bool mod_to);
};

template <class POP>
inline void TestProjector2D2<POP>::operate(size_t ie, size_t ip, 
					size_t from, size_t to, 
					real nx, real ny, 
					bool mod_from, bool mod_to)
{
  edge_dx = (PL->x(to) - PL->x(from))*0.5;
  edge_dy = (PL->y(to) - PL->y(from))*0.5;
  for (iv = 0; iv < VL->NumVars(); iv++) {
    //.. compute the averages (central variables)
    var_h[iv][ie] = .5*( var_point[iv][from] + var_point[iv][to]);
    //.. scalar multiplication of variable gradients with edge vector
    scal_from = edge_dx * gx[iv][from] + edge_dy * gy[iv][from];
    scal_to = edge_dx * gx[iv][to] + edge_dy * gy[iv][to];
    // Extrapolation of plus variables (first order reduced by central)
    var_to[iv][ie] = (1. - upw) * var_h[iv][ie] + upw * (var_point[iv][to] - scal_to);
    var_from[iv][ie] = (1. - upw) * var_h[iv][ie] + upw * (var_point[iv][from] + scal_from);
  };
  if ((PL->code(from) != 0) && (PL->code(to) != 0)) {
    size_t N_from = 0;
    for (size_t i = 0; i < POPL->NumFLinks(from); i++) {
      if (PL->code(POPL->FLinkPoint(from, i)) != 0) N_from++;
    };
    size_t N_to = 0;
    for (size_t i = 0; i < POPL->NumFLinks(to); i++) {
      if (PL->code(POPL->FLinkPoint(to, i)) != 0) N_to++;
    };
    if ((N_from == 2) || (N_to == 2)) {
      real vx = .5*nx;
      real vy = .5*ny;
      real pnx = BPL->Nx(from) + BPL->Nx(to);
      real pny = BPL->Ny(from) + BPL->Ny(to);
      TurnLeft(vx,vy);
      if (vx*pnx + vy*pny < 0) {
	vx = -vx;
	vy = -vy;
      };
      for (iv = 0; iv < VL->NumVars(); iv++) {
	var_h[iv][ie] += .5*((gx[iv][from]+gx[iv][to])*vx + (gy[iv][from]+gy[iv][to])*vy);
	var_from[iv][ie] += gx[iv][from]*vx + gy[iv][from]*vy;
	var_to[iv][ie] += gx[iv][to]*vx + gy[iv][to]*vy;
      };
      //cout << from << "->" << to << "  (" << vx << "," << vy << ")" << endl;
    };
  };
};

#include "TestProjector2D2.cc"

#endif
