// !!
// This is a part of MOUSE, a library for PDE's on unstructured grids
// Copyright (C) 1999 Oliver Gloth <oliver@vug.uni-duisburg.de>
// Institut fuer Verbrennung und Gasdynamik (Universitaet Duisburg)
// institute for combustion and gas dynamics (university of Duisburg)
// Thursday, 1 April, 1999 Duisburg, Germany
//
// please see http://www.vug.uni-duisburg.de/MOUSE for more information
// please send any questions or suggestions to mouse@www.vug.uni-duisburg.de
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// !!

#if !defined(VECTOR_H)
#define VECTOR_H

#include <iostream.h>
#include <stdlib.h>
#include <math.h>

extern unsigned short int i_reg,default_Dim;
extern double *dp1_reg,*dp2_reg;

class vector;
class matrix;

extern vector*** vector_array_3 (int,int,int,unsigned short int);
extern vector** vector_array_2 (int,int,unsigned short int);
extern vector* vector_array_1 (int,unsigned short int);

class vector {
  unsigned short int Dim;
  double *value;
public:
  vector ();
  vector (unsigned short int);
  vector (double,double);
  vector (double,double,double);
  vector (unsigned short int,double*);
  vector (const vector&);
  ~vector () {delete [] value;};
  
  double& operator[] (int i) {return value[i];};
  double operator[] (int i) const {return value[i];};
  void operator= (const vector&);

  double operator* (const vector&) const;
  vector operator+ (const vector&) const;
  vector operator- (const vector&) const;
  
  friend vector operator* (const vector&,const double);
  friend vector operator* (const double,const vector&);
  
  vector cross (const vector&);
  matrix dyad (const vector&);
  
  vector& operator*= (const double);
  vector& operator+= (const vector&);

  int dim () const {return Dim;};
  friend ostream& operator<< (ostream&,const vector);
};

class matrix {
  unsigned short int Dim1,Dim2;
  vector *row;
public:
  matrix (unsigned short int,unsigned short int);
  matrix (const matrix&);
  ~matrix () {delete [] row;};

  vector& operator[] (int i) {return row[i];};
  vector operator[] (int i) const {return row[i];};
  vector operator() (int) const;
  void operator= (const matrix&);
  
  vector operator* (const vector&) const;
  matrix operator+ (const matrix&) const;
  matrix operator- (const matrix&) const;
  matrix operator* (const matrix&) const;

  friend matrix operator* (const double,const matrix&);

  void make_zero (int r,int c,int dir);
  matrix inv ();
  matrix T ();

  int dim1 () const {return Dim1;};
  int dim2 () const {return Dim2;};
  friend ostream& operator<< (ostream&,const matrix);
};



inline
vector& vector::operator*= (const double a)
{
    dp1_reg = value;
    for (i_reg=0;i_reg<Dim;i_reg++)
	*dp1_reg++ *= a;
    return *this;
};

inline
vector& vector::operator+= (const vector& v)
{
    dp1_reg = value;
    dp2_reg = v.value;
    for (i_reg=0;i_reg<Dim;i_reg++)
	*dp1_reg++ += *dp2_reg++;
    return *this;
};





#endif







